/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.agzs.tls;

import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse fr die spezielle Umsetzung der Anzeige-Informationen fr BW D-Zeile A8.
 * Diese Klasse wird ber die Angabe des UmsetzungsModul verwendet und basiert auf der Default-Abarbeitung.
 * Es werden nur die Methoden mit der speziellen Abarbeitung berschrieben.
 * Umsetzung KExTls nach AnzeigenGlobal und Umsetzung AnzeigenGlobal nach KExTls.
 *
 * @version $Revision: 1.1 $ / $Date: 2012/05/23 15:47:43 $ / ($Author: Drapp $)
 *
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 *
 */
public class AnzeigeTypUmsetzungTlsBw1DZeile extends AnzeigeTypUmsetzungTlsRp1
{
  /**
   * DebugLogger fr Debug-Ausgaben.
   */
  private static Debug m_debug = Debug.getLogger();

  /**
   * Konstruktor mit bergabe der notwendigen HashMaps.
   * 
   * @param listeAnzeigeAktorTlsDes Liste mit AnzeigeAktorTls mit Key DEs.
   * @param listeAnzeigeAktorTlsAnzeige Liste mit AnzeigeQuerschnittAktorTls mit Key Anzeige.
   * @param listeAnzeigeEigenschaftIst Liste mit Attributgruppe atg.anzeigeEigenschaftIst fr alle Anzeigen aktuell.
   * @param listeAnzeigeStatus Liste mit Status Methoden fr alle Anzeigen.
   */
  public AnzeigeTypUmsetzungTlsBw1DZeile(HashMap listeAnzeigeAktorTlsDes, HashMap listeAnzeigeAktorTlsAnzeige, HashMap listeAnzeigeEigenschaftIst, HashMap listeAnzeigeStatus)
  {
    super(listeAnzeigeAktorTlsDes, listeAnzeigeAktorTlsAnzeige, listeAnzeigeEigenschaftIst, listeAnzeigeStatus);
  }
  
  /**
   * Konstruktor mit bergabe der notwendigen HashMaps und speziellen Anzeige-Informationen.
   * 
   * @param listeAnzeigeAktorTlsDes Liste mit AnzeigeAktorTls mit Key DEs.
   * @param listeAnzeigeAktorTlsAnzeige Liste mit AnzeigeQuerschnittAktorTls mit Key Anzeige.
   * @param kannFreieGrafik <code>true</code>, wenn Anzeige freie Grafik darstellen kann, sonst <code>false</code>.
   * @param kannFreiText <code>true</code>, wenn Anzeige freien Text darstellen kann, sonst <code>false</code>.
   */
  public AnzeigeTypUmsetzungTlsBw1DZeile(HashMap listeAnzeigeAktorTlsDes, HashMap listeAnzeigeAktorTlsAnzeige, boolean kannFreieGrafik, boolean kannFreiText)
  {
    super(listeAnzeigeAktorTlsDes, listeAnzeigeAktorTlsAnzeige, kannFreieGrafik, kannFreiText);
  }
  
  /**
   * Abarbeitung der TLS-WzgGrundEinstellung entsprechend den D-Zeilen-Vorgaben BW.
   * 
   * D.h. in Stellzustand bei Text die ersten 6 Zeichen Zusatzinformationen und dann folgen die sichtbaren Zeichen
   * 
   * @param result Attributgruppe atg.tlsWzgGrundEinstellung.
   */
  public Data tlsWzgGrundEinstellung(ResultData result)
  {
//  	System.out.println("AnzeigeTypUmsetzungTlsBw1DZeile.tlsWzgGrundEinstellung()");
    
    SystemObject de = result.getObject();
    
    AnzeigeAktorTls anzeigeAktor = (AnzeigeAktorTls)getListeAnzeigeAktorTlsDes().get(de.getPid());

    if (anzeigeAktor == null)
      return null;
    
    String anzeigePid = anzeigeAktor.getAnzeige().getPid();
    
    Data dataAnzeigeAutarkIst = AnzeigenGlobalZwischenSchichtTls.atgAnzeigeAutarkIstErstellen(AnzeigenGlobalZwischenSchichtTls.getConnection());
    
    m_debug.finer("tlsWzgGrundEinstellung DE " + de.getPid() + "\nAnzeige " + anzeigePid + "\nresult " + result + "\nData " + dataAnzeigeAutarkIst);

    if (result.isNoSourceAvailable()) // keine Quelle vorhanden
    {
      AnzeigenGlobalZwischenSchichtTls.atgAnzeigeAutarkIstInitialisieren(dataAnzeigeAutarkIst, result.getDataTime(), "Ja");
    }
    else // Quelle vorhanden
    {
      if (result.hasData()) // Daten vorhanden
      {
        Data data = result.getData();

        AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls = anzeigeAktor.getAnzeigeTypUmsetzung();

        String prinzip = data.getScaledValue("AnzeigePrinzip").getValueText();
        
        if ((prinzip.equals("c")) || (prinzip.equals("d")))
        {
          if (data.getArray("StellCode").getLength() == 1)
          {
            int code = data.getArray("StellCode").getItem(0).getScaledValue("Code").intValue();

            m_debug.finest("Code " + code);

            if (code == 0) // freie TextDefinition
            {
              {
                // Ergebnis lschen    
                
              	dataAnzeigeAutarkIst.getItem("Autark").getReferenceValue("AnzeigeInhalt").setSystemObject(null);
              	dataAnzeigeAutarkIst.getItem("Autark").getReferenceValue("Wechseltext").setSystemObject(null);
              	dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getReferenceValue("Font").setSystemObject(null);
              	dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getUnscaledValue("TextAusrichtung").set(-1);
              	dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").setLength(0);
              
                String zeichenfolge = "";
  
                for (int i = 0; i < data.getArray("TextDefinition").getLength(); i++)
                {
                  zeichenfolge += data.getArray("TextDefinition").getItem(i).getTextValue("Text").getValueText();
                }
                
                int laengeInput = zeichenfolge.length();
                int laengeText = laengeInput-6;
          
                if (laengeInput >= 6)
                {
                  int codeTextAusrichtung = zeichenfolge.codePointAt(0);
                  
                  // fr Tests Font ber ASCII 1, 2
                  
                  if (codeTextAusrichtung == 49)
                  {
                    codeTextAusrichtung = 128;
                  }
                  else if (codeTextAusrichtung == 50)
                  {
                    codeTextAusrichtung = 129;
                  }
                  
                  String pidFontUmsetzungTls = fontUmsetzungAusTlsBestimmen(anzeigeTypUmsetzungTls, codeTextAusrichtung, null);
          
                  FontUmsetzungTls fontUmsetzungTls = null;
      
                  if (pidFontUmsetzungTls != null)
                  {
                    HashMap mapFont = anzeigeTypUmsetzungTls.getListeFontUmsetzungTls();
              
                    if (mapFont != null)
                      fontUmsetzungTls = (FontUmsetzungTls)mapFont.get(pidFontUmsetzungTls);
                  }
      
                  HashMap mapZeichen = null;
            
                  if (fontUmsetzungTls != null)
                    mapZeichen = fontUmsetzungTls.getListeZeichenUmsetzungTlsCode();
      
                  // hier mit Fehler abbrechen, wenn kein Font bestimmt !!      
                  
                  if ((mapZeichen == null) || (mapZeichen.size() == 0))
                  {
                  	tlsWzgBlinken(data.getArray("StellCode").getItem(0), dataAnzeigeAutarkIst, "Autark");
                    dataAnzeigeAutarkIst.getScaledValue("Kommunikationsausfall").setText("Nein");

                    m_debug.warning(de.getPid() + "Font ber Code " + codeTextAusrichtung + " nicht bestimmbar");

                    return dataAnzeigeAutarkIst;
                  }
                  
                  // Pid Font ber Umsetzung bestimmen
          
                  String pidFont = null;
            
                  if (fontUmsetzungTls != null)
                  {
                    pidFont = fontUmsetzungTls.getPidFont();
                  }
              
                  if (laengeText < 0)
                  {
                    laengeText = 0;
        
                    m_debug.warning(de.getPid() + "Wechseltext zu kurz (6 + Text) '" + zeichenfolge + "'");
                
                    return null;
                  }
              
                  if (pidFont != null)
                  {
                  	dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getReferenceValue("Font").setText(pidFont);
                  }

                  // Textausrichtung aus Rckmeldung bestimmen
      
                  codeTextAusrichtung = zeichenfolge.codePointAt(1);
                  
                  // fr Tests Ausrichtung ber ASCII 1, 2, 3
                  
                  if (codeTextAusrichtung == 49)
                  {
                    codeTextAusrichtung = 133;
                  }
                  else if (codeTextAusrichtung == 50)
                  {
                    codeTextAusrichtung = 134;
                  }
                  else if (codeTextAusrichtung == 51)
                  {
                    codeTextAusrichtung = 135;
                  }

                  String textAusrichtung = textAusrichtungAusTlsBestimmen(anzeigeTypUmsetzungTls, codeTextAusrichtung);
                  
                  System.out.println("codeTextAusrichtung " + codeTextAusrichtung + " textAusrichtung " + textAusrichtung);
              
                  dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getUnscaledValue("TextAusrichtung").setText(textAusrichtung);
            
                  // Text        
            
                  dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").setLength(laengeText);
      
                  for (int i = 6; i < zeichenfolge.length(); i++)
                  {
                    m_debug.finest("Zeichen " + i + " " + zeichenfolge.codePointAt(i));
          
                    int codeZeichen = zeichenfolge.codePointAt(i);
          
                    ZeichenUmsetzungTls zeichenUmsetzungTls = (ZeichenUmsetzungTls)mapZeichen.get(codeZeichen);
                  
                    if (zeichenUmsetzungTls != null)
                    	dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").getReferenceValue(i-6).setText(zeichenUmsetzungTls.getPidZeichen());
                    else
                    	dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").getReferenceValue(i-6).setSystemObject(null);
                  }
                  
                  dataAnzeigeAutarkIst.getScaledValue("Kommunikationsausfall").setText("Nein");
                  tlsWzgBlinken(data.getArray("StellCode").getItem(0), dataAnzeigeAutarkIst, "Autark");
                }
                else // fehlerhafte Lnge, wenn grer als 0, wird mindestens 6 erwartet
                {
                  dataAnzeigeAutarkIst.getScaledValue("Kommunikationsausfall").setText("Ja");
                  tlsWzgBlinken(null, dataAnzeigeAutarkIst, "Autark");
                }
              }
            }
            else // code != 0
            {
            	if (code == m_codeZustandBleibtErhalten)
            	{
                dataAnzeigeAutarkIst.getItem("Autark").getReferenceValue("AnzeigeInhalt").setText(AnzeigenGlobalZwischenSchichtTls.getWvzInhaltBeibehalten());
                dataAnzeigeAutarkIst.getScaledValue("Kommunikationsausfall").setText("Nein");
                
                tlsWzgBlinken(null, dataAnzeigeAutarkIst, "Autark");
                
                // Wechseltextinformationen lschen
                
                dataAnzeigeAutarkIst.getItem("Autark").getReferenceValue("Wechseltext").setSystemObject(null);
                dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getReferenceValue("Font").setSystemObject(null);
                dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getUnscaledValue("TextAusrichtung").set(-1);
                dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").setLength(0);
            	}
            	else
            	{
                dataAnzeigeAutarkIst.getItem("Autark").getReferenceValue("AnzeigeInhalt").setSystemObject(null);
                dataAnzeigeAutarkIst.getScaledValue("Kommunikationsausfall").setText("Ja");
                
                tlsWzgBlinken(null, dataAnzeigeAutarkIst, "Autark");
                
                // Wechseltextinformationen lschen
                
                dataAnzeigeAutarkIst.getItem("Autark").getReferenceValue("Wechseltext").setSystemObject(null);
                dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getReferenceValue("Font").setSystemObject(null);
                dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getUnscaledValue("TextAusrichtung").set(-1);
                dataAnzeigeAutarkIst.getItem("Autark").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").setLength(0);
            	}
            }
          }
          else
          {
            tlsWzgBlinken(null, dataAnzeigeAutarkIst, "Autark");

            dataAnzeigeAutarkIst.getScaledValue("Kommunikationsausfall").setText("Ja");
          }
        }
      }
      else // keine Daten vorhanden
      {
        tlsWzgBlinken(null, dataAnzeigeAutarkIst, "Autark");

        dataAnzeigeAutarkIst.getScaledValue("Kommunikationsausfall").setText("Ja");
      }
    }
    
    return dataAnzeigeAutarkIst;
  }  

  /**
   * Abarbeitung der Autark-Versorgung aus dem Modell AnzeigenGlobal.
   * 
   * @param resultData AttributGruppe atg.anzeigeAutarkSoll.
   * @param zeitpunktUebergeordnet Zeitpunkt der bergeordneten Schaltung.
   * @return Befehl fr das Modell KExTls.
   */
  public ResultData anzeigeAutarkSoll(ResultData resultData, Long zeitpunktUebergeordnet)
  {
//  	System.out.println("AnzeigeTypUmsetzungTlsBw1DZeile.anzeigeAutarkSoll()");
  	
    ResultData ergebnis = null;
    
    SystemObject objekt = resultData.getObject();
    
    AnzeigeAktorTls anzeigeAktor = (AnzeigeAktorTls)getListeAnzeigeAktorTlsAnzeige().get(objekt.getPid());

    if (anzeigeAktor == null)
      return null;
    
    if (resultData.isNoSourceAvailable()) // keine Quelle vorhanden
    {
      return null;
    }
    else // Quelle vorhanden
    {
      if (resultData.hasData()) // Daten vorhanden
      {
        Data data = resultData.getData();
    
        AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls = anzeigeAktor.getAnzeigeTypUmsetzung();
    
        long zeitpunkt = AnzeigenGlobalZwischenSchichtTls.getConnection().getTime();

        // BefehlsArt prfen
        
        int befehlsArt = AnzeigeSollBefehlsArt.autarkBefehlsArtBestimmen(data);
        
        if (befehlsArt == AnzeigeSollBefehlsArt.WvzInhalt)
        {
        }
        else if (befehlsArt == AnzeigeSollBefehlsArt.Grafik)
        {
          if (isKannFreieGrafik()) // Grafik-Ansteuerung ok
          {
            
          }
          else
          {
            m_debug.error("Anzeige " + resultData.getObject().getPid() + " kann keine Grafik-Ansteuerung");
            
            return null;
          }
        }
        else if (befehlsArt == AnzeigeSollBefehlsArt.Wechseltext)
        {
          if (isKannFreiText()) // Wechseltext-Ansteuerung ok
          {
            
          }
          else
          {
            m_debug.error("Anzeige " + resultData.getObject().getPid() + " kann keine Wechseltext-Ansteuerung");
            
            return null;
          }
        }
        else
        {
          m_debug.error("ATG fr Anzeige " + resultData.getObject().getPid() + " beinhaltet keinen Befehl");
          
          return null;
        }
        
        if (befehlsArt == AnzeigeSollBefehlsArt.WvzInhalt) // reine Stellcode-Ansteuerung
        {
          HashMap mapWvzInhalt = null;
          
          ConfigurationObject objektAnzeigeInhalt = (ConfigurationObject)data.getItem("Autark").getReferenceValue("AnzeigeInhalt").getSystemObject();
          
          AttributeGroup atgWvzInhalt = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup("atg.wvzInhalt");
          
          Data dataWvzInhalt = objektAnzeigeInhalt.getConfigurationData(atgWvzInhalt);
          String bildinhalt = dataWvzInhalt.getTextValue("Bildinhalt").getValueText();
          
          m_debug.finest("Bildinhalt " + bildinhalt);

          // DEs bestimmen
          
          int anzahlDes = anzeigeAktor.getDes().size();
          
          if (anzahlDes == 1)
          {
            mapWvzInhalt = anzeigeTypUmsetzungTls.getListeWvzInhaltUmsetzungTlsWvzInhalt();
            
            WvzInhaltUmsetzungTls wvzInhaltUmsetzungTls = (WvzInhaltUmsetzungTls)mapWvzInhalt.get(bildinhalt);
  
            int code = -1;
            
            if (wvzInhaltUmsetzungTls != null)
            {
              code = wvzInhaltUmsetzungTls.getCode();
            }
            else
            {
            	if (bildinhalt.equals(AnzeigenGlobalZwischenSchichtTls.getBildinhaltBeibehalten()))
            	{
            		code = m_codeZustandBleibtErhalten;
            	}
            	else
            	{
            		m_debug.error("Objekt " + resultData.getObject().getPid() + " WvzInhalt " + bildinhalt + " nicht gefunden");
            	}
            }
            
            Set set = anzeigeAktor.getDes().keySet();
            
            Iterator it = set.iterator();
            
            String key = (String)it.next();
            
            SystemObject de = anzeigeAktor.getDes().get(key);
            
            int anzahlGlobalUmsetzung = anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().size();

            for (int j = 0; j < anzahlGlobalUmsetzung; j++)
            {
              GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().get(j);

              for (int k = 0; k < globalUmsetzung.getVerwendeteAttributGruppenAspekte().getAttributGruppenAspekte().size(); k++)
              {
                GlobalUmsetzungVerwendeteAttributGruppenAspekte guvaga = globalUmsetzung.getVerwendeteAttributGruppenAspekte();
                
                String atg = guvaga.getAttributGruppenAspekte().get(k).getPidAttributGruppe();
                String asp = guvaga.getAttributGruppenAspekte().get(k).getPidSendAspekt();
                String anzeigePrinzip = guvaga.getAttributGruppenAspekte().get(k).getAnzeigePrinzip();
                
                if (asp.length() > 0)
                {
                  if (atg.equals("atg.tlsWzgGrundEinstellung"))
                  {
                    AttributeGroup atgErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup(atg);
                    Aspect aspErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAspect(asp);
                    
                    Data dataErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().createData(AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup(atg));
                    
                    dataErgebnis.getScaledValue("AnzeigePrinzip").setText(anzeigePrinzip);
                      
                    dataErgebnis.getArray("WVZAnzeige").setLength(0);
                    dataErgebnis.getArray("StellCode").setLength(1);
                    
                    dataErgebnis.getArray("TextDefinition").setLength(0);
                    dataErgebnis.getArray("GruppenDefinition").setLength(0);
                    
                    dataErgebnis.getArray("StellCode").getItem(0).getUnscaledValue("Code").set(code);
                      
                    // BlinkenStatus
                    
                    String blinkenStatus = data.getItem("Autark").getScaledValue("Blinken").getValueText().equals("Ja") ? "ein" : "aus";
                    dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Status").setText(blinkenStatus);
  
                    // BlinkenDauer
                    
                    if (blinkenStatus.equals("ein"))
                    {
                      if (data.getItem("Autark").getScaledValue("Blinktakt").isState())
                      {
                        dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Dauer").set(5); // Default
                      }
                      else
                      {
                        double blinkenDauerAnzeige = data.getItem("Autark").getScaledValue("Blinktakt").doubleValue();
                        if (blinkenDauerAnzeige < 1.0)
                        {
                          blinkenDauerAnzeige = 1.0;
                        }
                        if (blinkenDauerAnzeige > 3.0)
                        {
                          blinkenDauerAnzeige = 3.0;
                        }
                        int blinkenDauerTls = (int)(blinkenDauerAnzeige*10)/2;
                        
                        dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Dauer").set(blinkenDauerTls);
                      }
                    }
                    else
                    {
                      dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Dauer").setText("aus");
                    }

                    dataErgebnis.getArray("StellCode").getItem(0).getItem("Status").getScaledValue("Fehler").setText("ok");
                    dataErgebnis.getArray("StellCode").getItem(0).getItem("Status").getScaledValue("Programm").setText("abgeschlossen");
                      
                    ergebnis = new ResultData(de, new DataDescription(atgErgebnis, aspErgebnis), zeitpunkt, dataErgebnis);
                    
                    break;
                  }
                }
              }
            }
          }
        }
        else
        {
          if (befehlsArt == AnzeigeSollBefehlsArt.Wechseltext) // reine Textansteuerung
          {
            ConfigurationObject wechseltext = (ConfigurationObject)data.getItem("Autark").getReferenceValue("Wechseltext").getSystemObject();

            Data dataWechseltext = null;
            
            if (wechseltext != null) // konfigurierter Wechseltext
            {
              AttributeGroup atgWechseltext = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup("atg.wechseltext");
              dataWechseltext = wechseltext.getConfigurationData(atgWechseltext);
            }
            else // dynamischer Wechseltext
            {
              dataWechseltext = data.getItem("Autark").getItem("DynamischerWechseltext");
            }
            
            String pidFont = dataWechseltext.getReferenceValue("Font").getValueText();
            int codeFont = fontUmsetzungAusAnzeigeBestimmen(anzeigeTypUmsetzungTls, pidFont);

            String pidTextAusrichtung = dataWechseltext.getScaledValue("TextAusrichtung").getValueText();
            int codeTextAusrichtung = textAusrichtungAusAnzeigeBestimmen(anzeigeTypUmsetzungTls, pidTextAusrichtung);

            HashMap mapZeichen = null;
            
            HashMap<String, FontUmsetzungTls> mapFont = anzeigeTypUmsetzungTls.getListeFontUmsetzungTls();
            
            if ((pidFont != null) && (mapFont != null))
            {
              TreeSet<String> treeset = new TreeSet<String>();
              treeset.addAll(mapFont.keySet());
            
              Iterator iterator = treeset.iterator();
            
              FontUmsetzungTls fontUmsetzungTls;
              
              while (iterator.hasNext())
              {
                fontUmsetzungTls = (FontUmsetzungTls)mapFont.get(iterator.next());
                
                if (fontUmsetzungTls.getPidFont().equals(pidFont))
                {
                  mapZeichen = fontUmsetzungTls.getListeZeichenUmsetzungTlsZeichen();
                  
                  break;
                }
              }
            }

            // DEs bestimmen
              
            int anzahlDes = anzeigeAktor.getDes().size();
            
            if (anzahlDes == 1) // Wechseltext nur mit einer DE
            {
              Set set = anzeigeAktor.getDes().keySet();
              
              Iterator it = set.iterator();
              
              String key = (String)it.next();
              
              SystemObject de = anzeigeAktor.getDes().get(key);
              
              int anzahlGlobalUmsetzung = anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().size();

              for (int j = 0; j < anzahlGlobalUmsetzung; j++)
              {
                GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().get(j);

                for (int k = 0; k < globalUmsetzung.getVerwendeteAttributGruppenAspekte().getAttributGruppenAspekte().size(); k++)
                {
                  GlobalUmsetzungVerwendeteAttributGruppenAspekte guvaga = globalUmsetzung.getVerwendeteAttributGruppenAspekte();
                  
                  String atg = guvaga.getAttributGruppenAspekte().get(k).getPidAttributGruppe();
                  String asp = guvaga.getAttributGruppenAspekte().get(k).getPidSendAspekt();
                  String anzeigePrinzip = guvaga.getAttributGruppenAspekte().get(k).getAnzeigePrinzip();

                  if (asp.length() > 0)
                  {
                    if (atg.equals("atg.tlsWzgGrundEinstellung"))
                    {
                      AttributeGroup atgErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup(atg);
                      Aspect aspErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAspect(asp);
                      
                      Data dataErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().createData(AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup(atg));

                      dataErgebnis.getScaledValue("AnzeigePrinzip").setText(anzeigePrinzip);
                      dataErgebnis.getArray("WVZAnzeige").setLength(0);
                      dataErgebnis.getArray("StellCode").setLength(1);
                      dataErgebnis.getArray("TextDefinition").setLength(0);
                      dataErgebnis.getArray("GruppenDefinition").setLength(0);

                      dataErgebnis.getArray("StellCode").getItem(0).getUnscaledValue("Code").set(0);

                      // BlinkenStatus
                      
                      String blinkenStatus = data.getItem("Autark").getScaledValue("Blinken").getValueText().equals("Ja") ? "ein" : "aus";
                      dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Status").setText(blinkenStatus);

                      // BlinkenDauer
                      
                      if (blinkenStatus.equals("ein"))
                      {
                        if (data.getItem("Autark").getScaledValue("Blinktakt").isState())
                        {
                          dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Dauer").set(5); // Default
                        }
                        else
                        {
                          double blinkenDauerAnzeige = data.getItem("Autark").getScaledValue("Blinktakt").doubleValue();
                          if (blinkenDauerAnzeige < 1.0)
                          {
                            blinkenDauerAnzeige = 1.0;
                          }
                          if (blinkenDauerAnzeige > 3.0)
                          {
                            blinkenDauerAnzeige = 3.0;
                          }
                          int blinkenDauerTls = (int)(blinkenDauerAnzeige*10)/2;
                          
                          dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Dauer").set(blinkenDauerTls);
                        }
                      }
                      else
                      {
                        dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Dauer").setText("aus");
                      }

                      dataErgebnis.getArray("StellCode").getItem(0).getItem("Status").getScaledValue("Fehler").setText("ok");
                      dataErgebnis.getArray("StellCode").getItem(0).getItem("Status").getScaledValue("Programm").setText("abgeschlossen");

                      ReferenceValue[] text = dataWechseltext.getReferenceArray("Textdefinition").getReferenceValues();

                      dataErgebnis.getArray("TextDefinition").setLength(6 + text.length);

                      int index = 0;
                      
                      dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", codeFont).toString());
                      dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", codeTextAusrichtung).toString());
                      dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", 0).toString());
                      dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", 0).toString());
                      dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", 0).toString());
                      dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", 0).toString());

                      int zeichen;
                      
                      for (int i = 0; i < text.length; i++)
                      {
                        String pid = text[i].getSystemObject().getPid();
                        
                        zeichen = 32; // Leerzeichen als default
                        
                        if (mapZeichen != null)
                        {
                          ZeichenUmsetzungTls zeichenUmsetzungTls = (ZeichenUmsetzungTls)mapZeichen.get(pid);
                          
                          if (zeichenUmsetzungTls != null)
                          {
                            zeichen = zeichenUmsetzungTls.getZeichencode();
                          }
                          else
                          {
                            zeichen = 32; // Leerzeichen als default
                          }
                        }

                        dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", zeichen).toString());
                      }

                      ergebnis = new ResultData(de, new DataDescription(atgErgebnis, aspErgebnis), zeitpunkt, dataErgebnis);
                      
                      break;
                    }
                  }
                }
              }
            }
            else // anzahlDes != 1
            {
              m_debug.error("Wechseltext nur mit einer DE mglich");
            }
          }
        }
      } 
      else // keine Daten vorhanden
      {
        return null;
      }
    }
    
    return ergebnis;
  }
}
