/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.agzs.tls;

import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;


import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse fr die spezielle Umsetzung der Anzeige-Informationen fr RP.
 * Diese Klasse wird ber die Angabe des UmsetzungsModul verwendet und basiert auf der Default-Abarbeitung.
 * Es werden nur die Methoden mit der speziellen Abarbeitung berschrieben.
 * Umsetzung KExTls nach AnzeigenGlobal und Umsetzung AnzeigenGlobal nach KExTls.
 *
 * @version $Revision: 1.4 $ / $Date: 2012/05/23 15:47:43 $ / ($Author: Drapp $)
 *
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 *
 */
public class AnzeigeTypUmsetzungTlsRp1 extends AnzeigeTypUmsetzungTlsDefault
{
  /**
   * DebugLogger fr Debug-Ausgaben.
   */
  private static Debug m_debug = Debug.getLogger();

  /**
   * Konstruktor mit bergabe der notwendigen HashMaps.
   * 
   * @param listeAnzeigeAktorTlsDes Liste mit AnzeigeAktorTls mit Key DEs.
   * @param listeAnzeigeAktorTlsAnzeige Liste mit AnzeigeQuerschnittAktorTls mit Key Anzeige.
   * @param listeAnzeigeEigenschaftIst Liste mit Attributgruppe atg.anzeigeEigenschaftIst fr alle Anzeigen aktuell.
   * @param listeAnzeigeStatus Liste mit Status Methoden fr alle Anzeigen.
   */
  public AnzeigeTypUmsetzungTlsRp1(HashMap listeAnzeigeAktorTlsDes, HashMap listeAnzeigeAktorTlsAnzeige, HashMap listeAnzeigeEigenschaftIst, HashMap listeAnzeigeStatus)
  {
    super(listeAnzeigeAktorTlsDes, listeAnzeigeAktorTlsAnzeige, listeAnzeigeEigenschaftIst, listeAnzeigeStatus);
  }
  
  /**
   * Konstruktor mit bergabe der notwendigen HashMaps und speziellen Anzeige-Informationen.
   * 
   * @param listeAnzeigeAktorTlsDes Liste mit AnzeigeAktorTls mit Key DEs.
   * @param listeAnzeigeAktorTlsAnzeige Liste mit AnzeigeQuerschnittAktorTls mit Key Anzeige.
   * @param kannFreieGrafik <code>true</code>, wenn Anzeige freie Grafik darstellen kann, sonst <code>false</code>.
   * @param kannFreiText <code>true</code>, wenn Anzeige freien Text darstellen kann, sonst <code>false</code>.
   */
  public AnzeigeTypUmsetzungTlsRp1(HashMap listeAnzeigeAktorTlsDes, HashMap listeAnzeigeAktorTlsAnzeige, boolean kannFreieGrafik, boolean kannFreiText)
  {
    super(listeAnzeigeAktorTlsDes, listeAnzeigeAktorTlsAnzeige, kannFreieGrafik, kannFreiText);
  }

  /**
   * Abarbeitung des TLS-StellZustand entsprechend den dWista-Vorgaben RP.
   * 
   * D.h. in Stellzustand bei Text die ersten 6 Zeichen Zusatzinformationen und dann folgen die sichtbaren Zeichen
   * 
   * @param result Attributgruppe atg.tlsWzgStellZustand
   */
  public Data tlsWzgStellZustand(ResultData result)
  {
    SystemObject object = result.getObject();
    
    DataModel dataModel = object.getDataModel();
    
    AnzeigeAktorTls anzeigeAktor = (AnzeigeAktorTls)getListeAnzeigeAktorTlsDes().get(object.getPid());

    if (anzeigeAktor == null)
      return null;
    
    String anzeigePid = anzeigeAktor.getAnzeige().getPid();
    
    Data dataAnzeigeEigenschaftIst = (Data)getListeAnzeigeEigenschaftIst().get(anzeigePid);
    
    m_debug.finer("tlsWzgStellZustand DE " + object.getPid() + "\nAnzeige " + anzeigePid + "\nresult " + result + "\nData " + dataAnzeigeEigenschaftIst);

    if (result.isNoSourceAvailable()) // keine Quelle vorhanden
    {
      AnzeigenGlobalZwischenSchichtTls.atgAnzeigeEigenschaftIstInitialisieren(dataAnzeigeEigenschaftIst, result.getDataTime(), statusBestimmen(anzeigePid, "tlsWzgStellZustand", STATUS_KOMMUNIKATIONSAUSFALL));
    }
    else // Quelle vorhanden
    {
      if (result.hasData()) // Daten vorhanden
      {
        Data data = result.getData();

        AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls = anzeigeAktor.getAnzeigeTypUmsetzung();

        String prinzip = data.getScaledValue("AnzeigePrinzip").getValueText();
        
        if ((prinzip.equals("a")) || (prinzip.equals("b"))) // WVZAnzeige
        {
          HashMap mapWvzInhalt = null;
          
          mapWvzInhalt = anzeigeTypUmsetzungTls.getListeWvzInhaltUmsetzungTlsCode();
      
          if (data.getArray("WVZAnzeige").getLength() == 1)
          {
            int code = data.getArray("WVZAnzeige").getItem(0).getScaledValue("Code").intValue();
            
            m_debug.finest("Code " + code);
            
            WvzInhaltUmsetzungTls wvzInhaltUmsetzungTls = (WvzInhaltUmsetzungTls)mapWvzInhalt.get(code);
            
            if (wvzInhaltUmsetzungTls != null)
            {
              bearbeiteWvzCodePlausibilitaet(anzeigeAktor, object, code, true, result.getDataTime());

              dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getReferenceValue("AnzeigeInhalt").setText(wvzInhaltUmsetzungTls.getPidWvzInhalt());
              dataAnzeigeEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigePid, "tlsWzgStellZustand", STATUS_OK));
            }
            else
            {
              bearbeiteWvzCodePlausibilitaet(anzeigeAktor, object, code, false, result.getDataTime());

              dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getReferenceValue("AnzeigeInhalt").setSystemObject(null);
              dataAnzeigeEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigePid, "tlsWzgStellZustand", STATUS_UNDEFINIERT));
            }
            
            tlsWzgStellZustandBlinken(data.getArray("WVZAnzeige").getItem(0), dataAnzeigeEigenschaftIst);
          }
          else
          {
            tlsWzgStellZustandBlinken(null, dataAnzeigeEigenschaftIst);

            dataAnzeigeEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigePid, "tlsWzgStellZustand", STATUS_UNDEFINIERT));
          }
        }
        else
        {
          if (data.getArray("StellCode").getLength() == 1)
          {
            int code = data.getArray("StellCode").getItem(0).getScaledValue("Code").intValue();
            
            m_debug.finest("Code " + code);

            if (((prinzip.equals("c")) || (prinzip.equals("d"))))
            {
              if (code == 0) // freie TextDefinition
              {
                // Ergebnis lschen    
  
                dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceValue("Font").setSystemObject(null);
                dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getUnscaledValue("TextAusrichtung").set(-1);
                dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").setLength(0);
              
                String zeichenfolge = "";
  
                for (int i = 0; i < data.getArray("TextDefinition").getLength(); i++)
                {
                  zeichenfolge += data.getArray("TextDefinition").getItem(i).getTextValue("Text").getValueText();
                }
                
                int laengeInput = zeichenfolge.length();
                int laengeText = laengeInput-6;
          
                if (laengeInput == 0) // Anzeige eingeschaltet, aber keine Steuerzeichen und keine Zeichen -> wie ausgeschaltet
                {
                  String textAusrichtung = anzeigeTypUmsetzungTls.getGrundTextAusrichtung();
                  String pidGrundFont = anzeigeTypUmsetzungTls.getPidGrundFont();
                  
                  Data dataDynamischerWechseltext = dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext");
                  
                  dataDynamischerWechseltext.getReferenceValue("Font").setSystemObjectPid(pidGrundFont, dataModel);
                  dataDynamischerWechseltext.getScaledValue("TextAusrichtung").setText(textAusrichtung);
                  dataDynamischerWechseltext.getReferenceArray("Textdefinition").setLength(0);
                  
                  dataAnzeigeEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigePid, "tlsWzgStellZustand", STATUS_OK));
                  
                  tlsWzgStellZustandBlinken(data.getArray("StellCode").getItem(0), dataAnzeigeEigenschaftIst);
                }
                else
                {  
                  if (laengeInput >= 6)
                  {
  
                    // Font aus Rckmeldung bestimmen, Grundfont nicht benutzen    
                
//                    String pidFontUmsetzungTls = fontUmsetzungAusTlsBestimmen(anzeigeTypUmsetzungTls, zeichenfolge.codePointAt(0), pidGrundFontUmsetzungTls);
                    
                    int codePointAt = zeichenfolge.codePointAt(0);
                    
                    // fr Tests Font ber ASCII 1, 2
                    
                    if (codePointAt == 49)
                    {
                      codePointAt = 128;
                    }
                    else if (codePointAt == 50)
                    {
                      codePointAt = 129;
                    }
                    
                    String pidFontUmsetzungTls = fontUmsetzungAusTlsBestimmen(anzeigeTypUmsetzungTls, codePointAt, null);
            
//                    System.out.println("pidFontUmsetzungTls " + pidFontUmsetzungTls);
                    
                    FontUmsetzungTls fontUmsetzungTls = null;
        
                    if (pidFontUmsetzungTls != null)
                    {
                      HashMap mapFont = anzeigeTypUmsetzungTls.getListeFontUmsetzungTls();
                
                      if (mapFont != null)
                        fontUmsetzungTls = (FontUmsetzungTls)mapFont.get(pidFontUmsetzungTls);
                    }
        
                    HashMap mapZeichen = null;
              
                    if (fontUmsetzungTls != null)
                      mapZeichen = fontUmsetzungTls.getListeZeichenUmsetzungTlsCode();
        
                    // hier mit Fehler abbrechen, wenn kein Font bestimmt !!      
                    
                    if ((mapZeichen == null) || (mapZeichen.size() == 0))
                    {
                      tlsWzgWechselTextBlinken(data.getArray("StellCode").getItem(0), dataAnzeigeEigenschaftIst);
                      dataAnzeigeEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigePid, "tlsWzgStellZustand", STATUS_UNDEFINIERT));
                    
                      return dataAnzeigeEigenschaftIst;
                    }
                    
                    // Pid Font ber Umsetzung bestimmen
            
                    String pidFont = null;
              
                    if (fontUmsetzungTls != null)
                    {
                      pidFont = fontUmsetzungTls.getPidFont();
                    }
                
                    if (laengeText < 0)
                    {
                      laengeText = 0;
          
                      m_debug.warning(object.getPid() + "Wechseltext zu kurz (6 + Text) '" + zeichenfolge + "'");
                  
                      return null;
                    }
                
                    if (pidFont != null)
                    {
                      dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceValue("Font").setText(pidFont);
                    }
  
                    // Textausrichtung aus Rckmeldung bestimmen
        
                    codePointAt = zeichenfolge.codePointAt(1);
                    
                    // fr Tests Ausrichtung ber ASCII 1, 2, 3
                    
                    if (codePointAt == 49)
                    {
                      codePointAt = 133;
                    }
                    else if (codePointAt == 50)
                    {
                      codePointAt = 134;
                    }
                    else if (codePointAt == 51)
                    {
                      codePointAt = 135;
                    }

                    String textAusrichtung = textAusrichtungAusTlsBestimmen(anzeigeTypUmsetzungTls, codePointAt);
                
                    dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getUnscaledValue("TextAusrichtung").setText(textAusrichtung);
              
                    // Text        
              
                    dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").setLength(laengeText);
        
                    for (int i = 6; i < zeichenfolge.length(); i++)
                    {
                      m_debug.finest("Zeichen " + i + " " + zeichenfolge.codePointAt(i));
            
                      int codeZeichen = zeichenfolge.codePointAt(i);
            
                      ZeichenUmsetzungTls zeichenUmsetzungTls = (ZeichenUmsetzungTls)mapZeichen.get(codeZeichen);
                    
                      if (zeichenUmsetzungTls != null)
                        dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").getReferenceValue(i-6).setText(zeichenUmsetzungTls.getPidZeichen());
                      else
                        dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext").getReferenceArray("Textdefinition").getReferenceValue(i-6).setSystemObject(null);
                    }
                    
                    dataAnzeigeEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigePid, "tlsWzgStellZustand", STATUS_OK));
                  }
                  else // fehlerhafte Lnge, wenn grer als 0, wird mindestens 6 erwartet
                  {
                    dataAnzeigeEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigePid, "tlsWzgStellZustand", STATUS_UNDEFINIERT));
                  }
                  
                  tlsWzgStellZustandBlinken(data.getArray("StellCode").getItem(0), dataAnzeigeEigenschaftIst);
                }
              }
              else if (code == -1) // aus
              {
                String textAusrichtung = anzeigeTypUmsetzungTls.getGrundTextAusrichtung();
                String pidGrundFont = anzeigeTypUmsetzungTls.getPidGrundFont();
                
                Data dataDynamischerWechseltext = dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getItem("DynamischerWechseltext");
                
                dataDynamischerWechseltext.getReferenceValue("Font").setSystemObjectPid(pidGrundFont, dataModel);
                dataDynamischerWechseltext.getScaledValue("TextAusrichtung").setText(textAusrichtung);
                dataDynamischerWechseltext.getReferenceArray("Textdefinition").setLength(0);

                dataAnzeigeEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigePid, "tlsWzgStellZustand", STATUS_OK));
                
                tlsWzgStellZustandBlinken(data.getArray("StellCode").getItem(0), dataAnzeigeEigenschaftIst);
              }
              else // in SST ber Code definierter Text
              {
                // hier noch keine Vorgehensweise festgelegt, Inhalte mssten ber Codedefinition abgefragt werden
              }
            }
          }
          else
          {
            tlsWzgStellZustandBlinken(null, dataAnzeigeEigenschaftIst);

            dataAnzeigeEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigePid, "tlsWzgStellZustand", STATUS_UNDEFINIERT));
          }
        }
      }
      else // keine Daten vorhanden
      {
        tlsWzgStellZustandBlinken(null, dataAnzeigeEigenschaftIst);

        dataAnzeigeEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(anzeigePid, "tlsWzgStellZustand", STATUS_UNDEFINIERT));
      }
    }
    
    return dataAnzeigeEigenschaftIst;
  }
  
  /**
   * Abarbeitung der Befehle aus dem Modell AnzeigenGlobal.
   * Speziell fr RP.
   * 
   * @param resultData AttributGruppe atg.anzeigeEigenschaft.
   * @param zeitpunktUebergeordnet Zeitpunkt der bergeordneten Schaltung.
   * @return Befehl fr das Modell KExTls.
   */
  public ResultData anzeigeEigenschaftSoll(ResultData resultData, Long zeitpunktUebergeordnet)
  {
    ResultData ergebnis = null;
    
    SystemObject objekt = resultData.getObject();
    
    AnzeigeAktorTls anzeigeAktor = (AnzeigeAktorTls)getListeAnzeigeAktorTlsAnzeige().get(objekt.getPid());

    if (anzeigeAktor == null)
      return null;
    
    if (resultData.isNoSourceAvailable()) // keine Quelle vorhanden
    {
      return null;
    }
    else // Quelle vorhanden
    {
      if (resultData.hasData()) // Daten vorhanden
      {
        Data data = resultData.getData();
    
        AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls = anzeigeAktor.getAnzeigeTypUmsetzung();
    
        long zeitpunkt = data.getTimeValue("SollZeitpunkt").getMillis();

        // BefehlsArt prfen
        
        int befehlsArt = AnzeigeSollBefehlsArt.eigenschaftBefehlsArtBestimmen(data);
        
        if (befehlsArt == AnzeigeSollBefehlsArt.WvzInhalt)
        {
          if ((!isKannFreieGrafik()) && (!isKannFreiText())) // reine Stellcode-Ansteuerung ok
          {
            
          }
          else
          {
            // TODO WvzInhalt ber Wechseltext !
            
            m_debug.error("Anzeige " + resultData.getObject().getPid() + " kann keine Stellcode-Ansteuerung");
            
            return null;
          }
        }
        else if (befehlsArt == AnzeigeSollBefehlsArt.Grafik)
        {
          if (isKannFreieGrafik()) // Grafik-Ansteuerung ok
          {
            
          }
          else
          {
            m_debug.error("Anzeige " + resultData.getObject().getPid() + " kann keine Grafik-Ansteuerung");
            
            return null;
          }
        }
        else if (befehlsArt == AnzeigeSollBefehlsArt.Wechseltext)
        {
          if (isKannFreiText()) // Wechseltext-Ansteuerung ok
          {
            
          }
          else
          {
            m_debug.error("Anzeige " + resultData.getObject().getPid() + " kann keine Wechseltext-Ansteuerung");
            
            return null;
          }
        }
        else
        {
          m_debug.error("ATG fr Anzeige " + resultData.getObject().getPid() + " beinhaltet keinen Befehl");
          
          return null;
        }
        
        if (befehlsArt == AnzeigeSollBefehlsArt.WvzInhalt) // reine Stellcode-Ansteuerung
        {
          HashMap mapWvzInhalt = null;
          
          ConfigurationObject objektAnzeigeInhalt = (ConfigurationObject)data.getItem("Eigenschaft").getReferenceValue("AnzeigeInhalt").getSystemObject();
          
          AttributeGroup atgWvzInhalt = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup("atg.wvzInhalt");
          
          Data dataWvzInhalt = objektAnzeigeInhalt.getConfigurationData(atgWvzInhalt);
          String anzeigeInhalt = dataWvzInhalt.getTextValue("Bildinhalt").getValueText();
          
          m_debug.finest("AnzeigeInhalt " + anzeigeInhalt);

          // DEs bestimmen
          
          int anzahlDes = anzeigeAktor.getDes().size();
          
          if (anzahlDes == 1)
          {
            mapWvzInhalt = anzeigeTypUmsetzungTls.getListeWvzInhaltUmsetzungTlsWvzInhalt();
            
            WvzInhaltUmsetzungTls wvzInhaltUmsetzungTls = (WvzInhaltUmsetzungTls)mapWvzInhalt.get(anzeigeInhalt);
  
            int code = -1;
            
            if (wvzInhaltUmsetzungTls != null)
            {
              code = wvzInhaltUmsetzungTls.getCode();
            }
            else
            {
              m_debug.error("Objekt " + resultData.getObject().getPid() + " WvzInhalt " + anzeigeInhalt + " nicht gefunden");
            }
            
            Set set = anzeigeAktor.getDes().keySet();
            
            Iterator it = set.iterator();
            
            String key = (String)it.next();
            
            SystemObject de = anzeigeAktor.getDes().get(key);
            
            int anzahlGlobalUmsetzung = anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().size();

            for (int j = 0; j < anzahlGlobalUmsetzung; j++)
            {
              GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().get(j);

              for (int k = 0; k < globalUmsetzung.getVerwendeteAttributGruppenAspekte().getAttributGruppenAspekte().size(); k++)
              {
                GlobalUmsetzungVerwendeteAttributGruppenAspekte guvaga = globalUmsetzung.getVerwendeteAttributGruppenAspekte();
                
                String atg = guvaga.getAttributGruppenAspekte().get(k).getPidAttributGruppe();
                String asp = guvaga.getAttributGruppenAspekte().get(k).getPidSendAspekt();
                String anzeigePrinzip = guvaga.getAttributGruppenAspekte().get(k).getAnzeigePrinzip();

                if (asp.length() > 0)
                {
                  if ((atg.equals("atg.tlsWzgWvzStellZustand")) ||
                      (atg.equals("atg.tlsWzgStellZustand")))
                  {
                    AttributeGroup atgErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup(atg);
                    Aspect aspErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAspect(asp);
                    
                    Data dataErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().createData(AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup(atg));

                    if (atg.equals("atg.tlsWzgWvzStellZustand"))
                    {
                      dataErgebnis.getUnscaledValue("WvzCode").set(code);

                      // BlinkenStatus
                      
                      String blinkenStatus = data.getItem("Eigenschaft").getScaledValue("Blinken").getValueText().equals("Ja") ? "ein" : "aus";
                      dataErgebnis.getItem("Blinken").getScaledValue("Status").setText(blinkenStatus);
                      
                      // BlinkenDauer
                      
                      if (blinkenStatus.equals("ein"))
                      {
                        if (data.getItem("Eigenschaft").getScaledValue("Blinktakt").isState())
                        {
                          dataErgebnis.getItem("Blinken").getScaledValue("Dauer").set(5); // Default
                        }
                        else
                        {
                          double blinkenDauerAnzeige = data.getItem("Eigenschaft").getScaledValue("Blinktakt").doubleValue();
                          if (blinkenDauerAnzeige < 1.0)
                          {
                            blinkenDauerAnzeige = 1.0;
                          }
                          if (blinkenDauerAnzeige > 3.0)
                          {
                            blinkenDauerAnzeige = 3.0;
                          }
                          int blinkenDauerTls = (int)(blinkenDauerAnzeige*10)/2;
                          
                          dataErgebnis.getItem("Blinken").getScaledValue("Dauer").set(blinkenDauerTls);
                        }
                      }
                      else
                      {
                        dataErgebnis.getItem("Blinken").getScaledValue("Dauer").setText("aus");
                      }
                      
                      if (zeitpunkt < ((new Date()).getTime()-600000000))
                      {
                        zeitpunkt = zeitpunktUebergeordnet.longValue();
                      }
                      
                      ergebnis = new ResultData(de, new DataDescription(atgErgebnis, aspErgebnis), zeitpunkt, dataErgebnis);
                    }
                    
                    if (atg.equals("atg.tlsWzgStellZustand"))
                    {
                      dataErgebnis.getScaledValue("AnzeigePrinzip").setText(anzeigePrinzip);
                      dataErgebnis.getArray("WVZAnzeige").setLength(1);
                      dataErgebnis.getArray("StellCode").setLength(0);
                      dataErgebnis.getArray("TextDefinition").setLength(0);
                      dataErgebnis.getArray("GruppenDefinition").setLength(0);
                      
                      dataErgebnis.getArray("WVZAnzeige").getItem(0).getUnscaledValue("Code").set(code);

                      // BlinkenStatus
                      
                      String blinkenStatus = data.getItem("Eigenschaft").getScaledValue("Blinken").getValueText().equals("Ja") ? "ein" : "aus";
                      dataErgebnis.getArray("WVZAnzeige").getItem(0).getItem("Blinken").getScaledValue("Status").setText(blinkenStatus);

                      // BlinkenDauer
                      
                      if (blinkenStatus.equals("ein"))
                      {
                        if (data.getItem("Eigenschaft").getScaledValue("Blinktakt").isState())
                        {
                          dataErgebnis.getArray("WVZAnzeige").getItem(0).getItem("Blinken").getScaledValue("Dauer").set(5); // Default
                        }
                        else
                        {
                          double blinkenDauerAnzeige = data.getItem("Eigenschaft").getScaledValue("Blinktakt").doubleValue();
                          if (blinkenDauerAnzeige < 1.0)
                          {
                            blinkenDauerAnzeige = 1.0;
                          }
                          if (blinkenDauerAnzeige > 3.0)
                          {
                            blinkenDauerAnzeige = 3.0;
                          }
                          int blinkenDauerTls = (int)(blinkenDauerAnzeige*10)/2;
                          
                          dataErgebnis.getArray("WVZAnzeige").getItem(0).getItem("Blinken").getScaledValue("Dauer").set(blinkenDauerTls);
                        }
                      }
                      else
                      {
                        dataErgebnis.getArray("WVZAnzeige").getItem(0).getItem("Blinken").getScaledValue("Dauer").setText("aus");
                      }

                      dataErgebnis.getArray("WVZAnzeige").getItem(0).getItem("Status").getScaledValue("Fehler").setText("ok");
                      dataErgebnis.getArray("WVZAnzeige").getItem(0).getItem("Status").getScaledValue("Programm").setText("abgeschlossen");
                      
                      if (zeitpunkt < ((new Date()).getTime()-600000000))
                      {
                        zeitpunkt = zeitpunktUebergeordnet.longValue();
                      }
                      
                      ergebnis = new ResultData(de, new DataDescription(atgErgebnis, aspErgebnis), zeitpunkt, dataErgebnis);
                    }
                    
                    break;
                  }
                }
              }
            }
          }
        }
        else
        {
          if (befehlsArt == AnzeigeSollBefehlsArt.Wechseltext) // reine Textansteuerung
          {
            ConfigurationObject wechseltext = (ConfigurationObject)data.getItem("Eigenschaft").getReferenceValue("Wechseltext").getSystemObject();

            Data dataWechseltext = null;
            
            if (wechseltext != null) // konfigurierter Wechseltext
            {
              AttributeGroup atgWechseltext = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup("atg.wechseltext");
              dataWechseltext = wechseltext.getConfigurationData(atgWechseltext);
            }
            else // dynamischer Wechseltext
            {
              dataWechseltext = data.getItem("Eigenschaft").getItem("DynamischerWechseltext");
            }
            
            String pidFont = dataWechseltext.getReferenceValue("Font").getValueText();
            int codeFont = fontUmsetzungAusAnzeigeBestimmen(anzeigeTypUmsetzungTls, pidFont);

            String pidTextAusrichtung = dataWechseltext.getScaledValue("TextAusrichtung").getValueText();
            int codeTextAusrichtung = textAusrichtungAusAnzeigeBestimmen(anzeigeTypUmsetzungTls, pidTextAusrichtung);

            HashMap mapZeichen = null;
            
            HashMap<String, FontUmsetzungTls> mapFont = anzeigeTypUmsetzungTls.getListeFontUmsetzungTls();
            
            if ((pidFont != null) && (mapFont != null))
            {
              TreeSet<String> treeset = new TreeSet<String>();
              treeset.addAll(mapFont.keySet());
            
              Iterator iterator = treeset.iterator();
            
              FontUmsetzungTls fontUmsetzungTls;
              
              while (iterator.hasNext())
              {
                fontUmsetzungTls = (FontUmsetzungTls)mapFont.get(iterator.next());
                
                if (fontUmsetzungTls.getPidFont().equals(pidFont))
                {
                  mapZeichen = fontUmsetzungTls.getListeZeichenUmsetzungTlsZeichen();
                  
                  break;
                }
              }
            }

            // DEs bestimmen
              
            int anzahlDes = anzeigeAktor.getDes().size();
            
            if (anzahlDes == 1) // Wechseltext nur mit einer DE
            {
              Set set = anzeigeAktor.getDes().keySet();
              
              Iterator it = set.iterator();
              
              String key = (String)it.next();
              
              SystemObject de = anzeigeAktor.getDes().get(key);
              
              int anzahlGlobalUmsetzung = anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().size();

              for (int j = 0; j < anzahlGlobalUmsetzung; j++)
              {
                GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)anzeigeTypUmsetzungTls.getListeGlobalUmsetzung().get(j);

                for (int k = 0; k < globalUmsetzung.getVerwendeteAttributGruppenAspekte().getAttributGruppenAspekte().size(); k++)
                {
                  GlobalUmsetzungVerwendeteAttributGruppenAspekte guvaga = globalUmsetzung.getVerwendeteAttributGruppenAspekte();
                  
                  String atg = guvaga.getAttributGruppenAspekte().get(k).getPidAttributGruppe();
                  String asp = guvaga.getAttributGruppenAspekte().get(k).getPidSendAspekt();
                  String anzeigePrinzip = guvaga.getAttributGruppenAspekte().get(k).getAnzeigePrinzip();

                  if (asp.length() > 0)
                  {
                    if ((atg.equals("atg.tlsWzgWechselText")) ||
                        (atg.equals("atg.tlsWzgStellZustand")))
                    {
                      AttributeGroup atgErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup(atg);
                      Aspect aspErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAspect(asp);
                      
                      Data dataErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().createData(AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup(atg));

                      if (atg.equals("atg.tlsWzgWechselText"))
                      {
                        StringBuffer befehl = new StringBuffer();
                        
                        befehl.append(String.format("%c", codeFont));
                        befehl.append(String.format("%c", codeTextAusrichtung));
                        befehl.append(String.format("%c", 0));
                        befehl.append(String.format("%c", 0));
                        befehl.append(String.format("%c", 0));
                        befehl.append(String.format("%c", 0));

                        ReferenceValue[] text = dataWechseltext.getReferenceArray("Textdefinition").getReferenceValues();

                        for (int i = 0; i < text.length; i++)
                        {
                          String pid = text[i].getSystemObject().getPid();
                          
                          if (mapZeichen == null)
                          {
                            continue;
                          }
                          
                          ZeichenUmsetzungTls zeichenUmsetzungTls = (ZeichenUmsetzungTls)mapZeichen.get(pid);
                          
                          if (zeichenUmsetzungTls != null)
                          {
                            befehl.append(String.format("%c", zeichenUmsetzungTls.getZeichencode()));
                          }
                          else
                          {
                            befehl.append(String.format("%c", 32)); // Leerzeichen als default
                          }
                        }

                        dataErgebnis.getTextValue("WVZTextZeichen").setText(befehl.toString());
                        
                        // BlinkenStatus
                        
                        String blinkenStatus = data.getItem("Eigenschaft").getScaledValue("Blinken").getValueText().equals("Ja") ? "ein" : "aus";
                        dataErgebnis.getItem("Blinken").getScaledValue("Status").setText(blinkenStatus);
                        
                        // BlinkenDauer
                        
                        if (blinkenStatus.equals("ein"))
                        {
                          if (data.getItem("Eigenschaft").getScaledValue("Blinktakt").isState())
                          {
                            dataErgebnis.getItem("Blinken").getScaledValue("Dauer").set(5); // Default
                          }
                          else
                          {
                            double blinkenDauerAnzeige = data.getItem("Eigenschaft").getScaledValue("Blinktakt").doubleValue();
                            if (blinkenDauerAnzeige < 1.0)
                            {
                              blinkenDauerAnzeige = 1.0;
                            }
                            if (blinkenDauerAnzeige > 3.0)
                            {
                              blinkenDauerAnzeige = 3.0;
                            }
                            int blinkenDauerTls = (int)(blinkenDauerAnzeige*10)/2;
                            
                            dataErgebnis.getItem("Blinken").getScaledValue("Dauer").set(blinkenDauerTls);
                          }
                        }
                        else
                        {
                          dataErgebnis.getItem("Blinken").getScaledValue("Dauer").setText("aus");
                        }

                        if (zeitpunkt < ((new Date()).getTime()-600000000))
                        {
                          zeitpunkt = zeitpunktUebergeordnet.longValue();
                        }
                        
                        ergebnis = new ResultData(de, new DataDescription(atgErgebnis, aspErgebnis), zeitpunkt, dataErgebnis);
                      }
                      
                      if (atg.equals("atg.tlsWzgStellZustand"))
                      {
                        dataErgebnis.getScaledValue("AnzeigePrinzip").setText(anzeigePrinzip);
                        dataErgebnis.getArray("WVZAnzeige").setLength(0);
                        dataErgebnis.getArray("StellCode").setLength(1);
                        dataErgebnis.getArray("TextDefinition").setLength(0);
                        dataErgebnis.getArray("GruppenDefinition").setLength(0);

                        dataErgebnis.getArray("StellCode").getItem(0).getUnscaledValue("Code").set(0);

                        // BlinkenStatus
                        
                        String blinkenStatus = data.getItem("Eigenschaft").getScaledValue("Blinken").getValueText().equals("Ja") ? "ein" : "aus";
                        dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Status").setText(blinkenStatus);

                        // BlinkenDauer
                        
                        if (blinkenStatus.equals("ein"))
                        {
                          if (data.getItem("Eigenschaft").getScaledValue("Blinktakt").isState())
                          {
                            dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Dauer").set(5); // Default
                          }
                          else
                          {
                            double blinkenDauerAnzeige = data.getItem("Eigenschaft").getScaledValue("Blinktakt").doubleValue();
                            if (blinkenDauerAnzeige < 1.0)
                            {
                              blinkenDauerAnzeige = 1.0;
                            }
                            if (blinkenDauerAnzeige > 3.0)
                            {
                              blinkenDauerAnzeige = 3.0;
                            }
                            int blinkenDauerTls = (int)(blinkenDauerAnzeige*10)/2;
                            
                            dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Dauer").set(blinkenDauerTls);
                          }
                        }
                        else
                        {
                          dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Dauer").setText("aus");
                        }

                        dataErgebnis.getArray("StellCode").getItem(0).getItem("Status").getScaledValue("Fehler").setText("ok");
                        dataErgebnis.getArray("StellCode").getItem(0).getItem("Status").getScaledValue("Programm").setText("abgeschlossen");

                        ReferenceValue[] text = dataWechseltext.getReferenceArray("Textdefinition").getReferenceValues();

                        dataErgebnis.getArray("TextDefinition").setLength(6 + text.length);

                        int index = 0;
                        
                        dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", codeFont).toString());
                        dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", codeTextAusrichtung).toString());
                        dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", 0).toString());
                        dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", 0).toString());
                        dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", 0).toString());
                        dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", 0).toString());

                        int zeichen;
                        
                        for (int i = 0; i < text.length; i++)
                        {
                          String pid = text[i].getSystemObject().getPid();
                          
                          zeichen = 32; // Leerzeichen als default
                          
                          if (mapZeichen != null)
                          {
                            ZeichenUmsetzungTls zeichenUmsetzungTls = (ZeichenUmsetzungTls)mapZeichen.get(pid);
                            
                            if (zeichenUmsetzungTls != null)
                            {
                              zeichen = zeichenUmsetzungTls.getZeichencode();
                            }
                            else
                            {
                              zeichen = 32; // Leerzeichen als default
                            }
                          }

                          dataErgebnis.getArray("TextDefinition").getItem(index++).getTextValue("Text").setText(String.format("%c", zeichen).toString());
                        }

                        if (zeitpunkt < ((new Date()).getTime()-600000000))
                        {
                          zeitpunkt = zeitpunktUebergeordnet.longValue();
                        }
                        
                        ergebnis = new ResultData(de, new DataDescription(atgErgebnis, aspErgebnis), zeitpunkt, dataErgebnis);
                      }
                      
                      break;
                    }
                  }
                }
              }
            }
            else // anzahlDes != 1
            {
              m_debug.error("Wechseltext nur mit einer DE mglich");
            }
          }
        }
      } 
      else // keine Daten vorhanden
      {
        return null;
      }
    }
    
    return ergebnis;
  }
  
  /**
   * Bestimmung der Text-Ausrichtung, kann sie nicht bestimmt werden, wird die Text-Ausrichtung 'Undefiniert' zurckgegeben.
   * 
   * @param anzeigeTypUmsetzungTls Objekt vom Typ AnzeigeTypUmsetzungTls.
   * @param code Zeichen aus Text von KExTls.
   * @return Text-Ausrichtung.
   */
  protected String textAusrichtungAusTlsBestimmen(AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls, int code)
  {
    String ergebnis = "Undefiniert";

    HashMap<String, TextAusrichtungUmsetzungTls> mapTextAusrichtung = anzeigeTypUmsetzungTls.getListeTextAusrichtungUmsetzungTls();
    
    if (mapTextAusrichtung != null)
    {
      TreeSet<String> treeset = new TreeSet<String>();
      treeset.addAll(mapTextAusrichtung.keySet());
    
      Iterator iterator = treeset.iterator();
    
      TextAusrichtungUmsetzungTls textAusrichtungUmsetzungTls;
      
      while (iterator.hasNext())
      {
        textAusrichtungUmsetzungTls = (TextAusrichtungUmsetzungTls)mapTextAusrichtung.get(iterator.next());
        
        if (code == textAusrichtungUmsetzungTls.getCodeTextAusrichtung())
        {
          ergebnis = textAusrichtungUmsetzungTls.getTextAusrichtung();

          break;
        }
      }
    }
    
    return ergebnis;
  }

  /**
   * Bestimmung des Code der Text-Ausrichtung, kann sie nicht bestimmt werden, wird Code 0 zurckgegeben.
   * 
   * @param anzeigeTypUmsetzungTls Objekt vom Typ AnzeigeTypUmsetzungTls.
   * @param pidAusrichtung Pid der TextAusrichtung
   * @return Code der TextAusrichtung
   */
  protected int textAusrichtungAusAnzeigeBestimmen(AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls, String pidAusrichtung)
  {
    int ergebnis = 133; // Linksbndig als Default

    HashMap<String, TextAusrichtungUmsetzungTls> mapTextAusrichtung = anzeigeTypUmsetzungTls.getListeTextAusrichtungUmsetzungTls();
    
    if ((pidAusrichtung != null) && (mapTextAusrichtung != null))
    {
      TreeSet<String> treeset = new TreeSet<String>();
      treeset.addAll(mapTextAusrichtung.keySet());
    
      Iterator iterator = treeset.iterator();
    
      TextAusrichtungUmsetzungTls textAusrichtungUmsetzungTls;
      
      while (iterator.hasNext())
      {
        textAusrichtungUmsetzungTls = (TextAusrichtungUmsetzungTls)mapTextAusrichtung.get(iterator.next());
        
        if (textAusrichtungUmsetzungTls.getTextAusrichtung().equals(pidAusrichtung))
        {
          ergebnis = textAusrichtungUmsetzungTls.getCodeTextAusrichtung();

          break;
        }
      }
    }
    
    return ergebnis;
  }

  /**
   * Bestimmung der Pid der FontUmsetzungTls, kann er nicht bestimmt werden, wird die Pid des GrundFontUmsetzungTls zurckgegeben.
   * 
   * @param anzeigeTypUmsetzungTls Objekt vom Typ AnzeigeTypUmsetzungTls.
   * @param code Zeichen aus Text von KExTls.
   * @param pidGrundFontUmsetzungTls Pid des GrundFonts der FontUmsetzungTls.
   * @return Pid der FontUmsetzungTls.
   */
  protected String fontUmsetzungAusTlsBestimmen(AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls, int code, String pidGrundFontUmsetzungTls)
  {
    String ergebnis = pidGrundFontUmsetzungTls;
    
    HashMap<String, FontUmsetzungTls> mapFont = anzeigeTypUmsetzungTls.getListeFontUmsetzungTls();
    
    if (mapFont != null)
    {
      TreeSet<String> treeset = new TreeSet<String>();
      treeset.addAll(mapFont.keySet());
    
      Iterator iterator = treeset.iterator();
    
      FontUmsetzungTls fontUmsetzungTls;
      
      while (iterator.hasNext())
      {
        fontUmsetzungTls = (FontUmsetzungTls)mapFont.get(iterator.next());
        
        if (code == fontUmsetzungTls.getCodeFont())
        {
          ergebnis = fontUmsetzungTls.getPid();
          
          break;
        }
      }
    }
    
    return ergebnis;
  }

  /**
   * Bestimmung des Code der FontUmsetzungTls, kann er nicht bestimmt werden, wird der Code 0 zurckgegeben.
   * 
   * @param anzeigeTypUmsetzungTls Objekt vom Typ AnzeigeTypUmsetzungTls.
   * @param pidFont Pid des Font
   * @return Code des Font
   */
  protected int fontUmsetzungAusAnzeigeBestimmen(AnzeigeTypUmsetzungTls anzeigeTypUmsetzungTls, String pidFont)
  {
    int ergebnis = 128; // Mittelschrift als Default

    HashMap<String, FontUmsetzungTls> mapFont = anzeigeTypUmsetzungTls.getListeFontUmsetzungTls();
    
    if ((pidFont != null) && (mapFont != null))
    {
      TreeSet<String> treeset = new TreeSet<String>();
      treeset.addAll(mapFont.keySet());
    
      Iterator iterator = treeset.iterator();
    
      FontUmsetzungTls fontUmsetzungTls;
      
      while (iterator.hasNext())
      {
        fontUmsetzungTls = (FontUmsetzungTls)mapFont.get(iterator.next());
        
        if (fontUmsetzungTls.getPidFont().equals(pidFont))
        {
          ergebnis = fontUmsetzungTls.getCodeFont();
          
          break;
        }
      }
    }
    
    return ergebnis;
  }
}
