/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.agzs.tls;

import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;
import java.util.TreeSet;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.Data.NumberArray;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.Data.ReferenceValue;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse fr die grundlegende (default) Umsetzung der Dirigent-Informationen.
 * Umsetzung KExTls nach AnzeigenGlobal und Umsetzung AnzeigenGlobal nach KExTls.
 *
 * @version $Revision: 1.1 $ / $Date: 2012/03/09 10:55:21 $ / ($Author: Drapp $)
 *
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 *
 */
public class DirigentUmsetzungTlsDefault
{  
  /**
   * DebugLogger fr Debug-Ausgaben.
   */
  private static Debug m_debug = Debug.getLogger();
  /**
   * Status ok.
   */
  final static int STATUS_OK = 0;
  /**
   * Status TeilStrung.
   */
  final static int STATUS_TEILSTOERUNG = 1;
  /**
   * Status gestrt.
   */
  final static int STATUS_GESTOERT = 2;
  /**
   * Status Stromausfall.
   */
  final static int STATUS_STROMAUSFALL = 3;
  /**
   * Status Trkontakt.
   */
  final static int STATUS_TRKONTAKT = 4;
  /**
   * Status undefiniert.
   */
  final static int STATUS_UNDEFINIERT = 5;
  /**
   * Status Kommunikationsausfall.
   */
  final static int STATUS_KOMMUNIKATIONSAUSFALL = 6;
  /**
   * Kanalsteuerung wird von atg.tlsGloDeFehler bernommen, daher muss atg.tlsGloKanalSteuerung nicht ausgewertet werden 
   */
  final static boolean m_kanalSteuerungUeberDeFehler = true;
  /**
   * HashMap mit DirigentAktorTls mit Key DEs.
   */
  protected HashMap m_listeDirigentAktorTlsDes = null;
  /**
   * HashMap mit DirigentAktorTls mit Key Dirigent.
   */
  protected HashMap m_listeDirigentAktorTlsDirigent = null;
  /**
   * HashMap mit Attributgruppe atg.dirigentEigenschaftIst fr alle Dirigenten aktuell.
   */
  protected HashMap m_listeDirigentEigenschaftIst = null;
  /**
   * HashMap fr Status Methoden fr alle Dirigenten.
   */
  protected HashMap m_listeDirigentStatus = null;
  
  /**
   * Konstruktor mit bergabe der notwendigen HashMaps.
   * 
   * @param listeDirigentAktorTlsDes Liste mit DirigentAktorTls mit Key DEs.
   * @param listeDirigentAktorTlsDirigent Liste mit DirigentAktorTls mit Key Dirigent.
   * @param listeDirigentEigenschaftIst Liste mit Attributgruppe atg.dirigentEigenschaftIst fr alle Dirigenten aktuell.
   * @param listeDirigentStatus Liste mit Status Methoden fr alle Dirigenten.
   */
  public DirigentUmsetzungTlsDefault(HashMap listeDirigentAktorTlsDes, HashMap listeDirigentAktorTlsDirigent, HashMap listeDirigentEigenschaftIst, HashMap listeDirigentStatus)
  {
    m_listeDirigentAktorTlsDes = listeDirigentAktorTlsDes;
    m_listeDirigentAktorTlsDirigent = listeDirigentAktorTlsDirigent;
    m_listeDirigentEigenschaftIst = listeDirigentEigenschaftIst;
    m_listeDirigentStatus = listeDirigentStatus;

    m_debug.finer("DirigentUmsetzungTlsDefault");
  }
  
  /**
   * Konstruktor mit bergabe der notwendigen HashMaps und speziellen Dirigent-Informationen.
   * 
   * @param listeDirigentAktorTlsDes Liste mit DirigentAktorTls mit Key DEs.
   * @param listeDirigentAktorTlsDirigent Liste mit DirigentAktorTls mit Key Dirigent.
   */
  public DirigentUmsetzungTlsDefault(HashMap listeDirigentAktorTlsDes, HashMap listeDirigentAktorTlsDirigent)
  {
    m_listeDirigentAktorTlsDes = listeDirigentAktorTlsDes;
    m_listeDirigentAktorTlsDirigent = listeDirigentAktorTlsDirigent;
    m_listeDirigentEigenschaftIst = null;
    m_listeDirigentStatus = null;

    m_debug.finer("DirigentUmsetzungTlsDefault");
  }
  
  /**
   * Abarbeitung des DE-Fehler genau nach TLS.
   * 
   * @param result Attributgruppe atg.tlsGloDeFehler.
   */
  public Data tlsGloDeFehler(ResultData result)
  {
    SystemObject object = result.getObject();
    
    DirigentAktorTls dirigentAktor = (DirigentAktorTls)getListeDirigentAktorTlsDes().get(object.getPid());

    if (dirigentAktor == null)
      return null;
    
    String dirigentPid = dirigentAktor.getDirigent().getPid();
    
    System.out.println("dirigentPid " + dirigentPid);
    
    Data dataDirigentEigenschaftIst = (Data)getListeDirigentEigenschaftIst().get(dirigentPid);

    System.out.println("dataDirigentEigenschaftIst " + dataDirigentEigenschaftIst);
    

    m_debug.finer("tlsGloDeFehler DE " + object.getPid() + "\nDirigent " + dirigentPid + "\nresult " + result + "\nData " + dataDirigentEigenschaftIst);

    if (result.isNoSourceAvailable()) // keine Quelle vorhanden
    {
      AnzeigenGlobalZwischenSchichtTls.atgDirigentEigenschaftIstInitialisieren(dataDirigentEigenschaftIst, result.getDataTime(), statusBestimmen(dirigentPid, "tlsGloDeFehler", STATUS_KOMMUNIKATIONSAUSFALL));
    }
    else // Quelle vorhanden
    {
      if (result.hasData()) // Daten vorhanden
      {
        Data data = result.getData();
  
        String deFehlerStatus = data.getTextValue("DEFehlerStatus").getValueText();
        
        if (deFehlerStatus.equals("ok"))
        {
          dataDirigentEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(dirigentPid, "tlsGloDeFehler", STATUS_OK));
//          dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getTimeValue("Zeit").setMillis(result.getDataTime());
        }
        else
        {
          AnzeigenGlobalZwischenSchichtTls.atgDirigentEigenschaftIstInitialisieren(dataDirigentEigenschaftIst, result.getDataTime(), statusBestimmen(dirigentPid, "tlsGloDeFehler", STATUS_GESTOERT));
        }
  
        String deKanalStatus = data.getTextValue("DEKanalStatus").getValueText();
        String passiviert = deKanalStatus.equals("aktiv") ? "Nein" : "Ja";
        dataDirigentEigenschaftIst.getItem("Eigenschaft").getScaledValue("Passiviert").setText(passiviert);
      }
      else // keine Daten vorhanden
      {
        AnzeigenGlobalZwischenSchichtTls.atgDirigentEigenschaftIstInitialisieren(dataDirigentEigenschaftIst, result.getDataTime(), statusBestimmen(dirigentPid, "tlsGloDeFehler", STATUS_UNDEFINIERT));
      }
    }
    
    return dataDirigentEigenschaftIst;
  }
 
  /**
   * Abarbeitung der Kanalsteuerung genau nach TLS.
   * Wird ber DE-Fehler abgearbeitet, deshalb ausser Betrieb.
   * 
   * @param result Attributgruppe atg.tlsGloKanalSteuerung.
   */
  public Data tlsGloKanalSteuerung(ResultData result)
  {
    if (m_kanalSteuerungUeberDeFehler)
      return null;
    
    SystemObject object = result.getObject();
    
    DirigentAktorTls dirigentAktor = (DirigentAktorTls)getListeDirigentAktorTlsDes().get(object.getPid());

    if (dirigentAktor == null)
      return null;
    
    String dirigentPid = dirigentAktor.getDirigent().getPid();
    
    Data dataDirigentEigenschaftIst = (Data)getListeDirigentEigenschaftIst().get(dirigentPid);

    if (dataDirigentEigenschaftIst == null)
      return null;
    
    m_debug.finer("tlsGloKanalSteuerung DE " + object.getPid() + "\nDirigent " + dirigentPid + "\nresult " + result + "\nData " + dataDirigentEigenschaftIst);
    
    if (result.isSourceAvailable()) // Quelle vorhanden
    {
      if (result.hasData()) // Daten vorhanden
      {
        Data data = result.getData();
  
        String deKanalStatus = data.getTextValue("DEKanalStatus").getValueText();
        String passiviert = deKanalStatus.equals("aktiv") ? "Nein" : "Ja";
        dataDirigentEigenschaftIst.getItem("Eigenschaft").getScaledValue("Passiviert").setText(passiviert);
      }
      else // keine Daten vorhanden
      {
        dataDirigentEigenschaftIst = null; // keine nderung melden
      }
    }
    
    return dataDirigentEigenschaftIst;
  }
  
  /**
   * Abarbeitung des TLS-WvzStellZustand nach TLS 93.
   * 
   * @param result Attributgruppe atg.tlsWzgWvzStellZustand.
   */
  public Data tlsWzgWvzStellZustand(ResultData result)
  {
    SystemObject object = result.getObject();
    
    DirigentAktorTls dirigentAktor = (DirigentAktorTls)getListeDirigentAktorTlsDes().get(object.getPid());

    if (dirigentAktor == null)
      return null;
    
    String dirigentPid = dirigentAktor.getDirigent().getPid();
    
    Data dataDirigentEigenschaftIst = (Data)getListeDirigentEigenschaftIst().get(dirigentPid);
    
    m_debug.finer("tlsWzgWvzStellZustand DE " + object.getPid() + "\nDirigent " + dirigentPid + "\nresult " + result + "\nData " + dataDirigentEigenschaftIst);

    if (result.isNoSourceAvailable()) // keine Quelle vorhanden
    {
      AnzeigenGlobalZwischenSchichtTls.atgDirigentEigenschaftIstInitialisieren(dataDirigentEigenschaftIst, result.getDataTime(), statusBestimmen(dirigentPid, "tlsWzgWvzStellZustand", STATUS_KOMMUNIKATIONSAUSFALL));
    }
    else // Quelle vorhanden
    {
//      dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getTimeValue("Zeit").setMillis(result.getDataTime());
      dataDirigentEigenschaftIst.getItem("Eigenschaft").getReferenceValue("AnzeigeInhalt").setSystemObject(null);
  
      if (result.hasData()) // Daten vorhanden
      {
        Data data = result.getData();
        
        DirigentUmsetzungTls dirigentUmsetzungTls = dirigentAktor.getDirigentUmsetzung();
    
        HashMap mapWvzInhalt = null;
    
        mapWvzInhalt = dirigentUmsetzungTls.getListeWvzInhaltUmsetzungTlsCode();
    
        int code = data.getScaledValue("WvzCode").intValue();
      
        m_debug.finest("Code " + code);

        WvzInhaltUmsetzungTls wvzInhaltUmsetzungTls = (WvzInhaltUmsetzungTls)mapWvzInhalt.get(code);
        
        System.out.println("wvzInhaltUmsetzungTls " + wvzInhaltUmsetzungTls);
        
        if (wvzInhaltUmsetzungTls != null)
        {
          dataDirigentEigenschaftIst.getItem("Eigenschaft").getReferenceValue("AnzeigeInhalt").setText(wvzInhaltUmsetzungTls.getPidWvzInhalt());
        }

        tlsWzgWvzStellZustandBlinken(data, dataDirigentEigenschaftIst);

        dataDirigentEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(dirigentPid, "tlsWzgWvzStellZustand", STATUS_OK));
      } 
      else // keine Daten vorhanden
      {
        dataDirigentEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(dirigentPid, "tlsWzgWvzStellZustand", STATUS_UNDEFINIERT));
      }
    }
    
    return dataDirigentEigenschaftIst;
  }
  
  /**
   * Abarbeitung des TLS-WvzStellZustand nach TLS 2002.
   * 
   * @param result Attributgruppe atg.tlsWzgStellZustand.
   */
  public Data tlsWzgStellZustand(ResultData result)
  {
    System.out.println("DirigentUmsetzungTlsDefault.tlsWzgStellZustand() " + result);
    
    SystemObject object = result.getObject();
    
    DirigentAktorTls dirigentAktor = (DirigentAktorTls)getListeDirigentAktorTlsDes().get(object.getPid());

    if (dirigentAktor == null)
      return null;
    
    String dirigentPid = dirigentAktor.getDirigent().getPid();
    
    Data dataDirigentEigenschaftIst = (Data)getListeDirigentEigenschaftIst().get(dirigentPid);
    
    m_debug.finer("tlsWzgStellZustand DE " + object.getPid() + "\nDirigent " + dirigentPid + "\nresult " + result + "\nData " + dataDirigentEigenschaftIst);

    if (result.isNoSourceAvailable()) // keine Quelle vorhanden
    {
      AnzeigenGlobalZwischenSchichtTls.atgDirigentEigenschaftIstInitialisieren(dataDirigentEigenschaftIst, result.getDataTime(), statusBestimmen(dirigentPid, "tlsWzgStellZustand", STATUS_KOMMUNIKATIONSAUSFALL));
    }
    else // Quelle vorhanden
    {
      if (result.hasData()) // Daten vorhanden
      {
        Data data = result.getData();

        DirigentUmsetzungTls dirigentUmsetzungTls = dirigentAktor.getDirigentUmsetzung();

        String prinzip = data.getScaledValue("AnzeigePrinzip").getValueText();
        
        if (prinzip.equals("Cluster"))
        {
          HashMap mapWvzInhalt = null;
          
          mapWvzInhalt = dirigentUmsetzungTls.getListeWvzInhaltUmsetzungTlsCode();
      
          if (data.getArray("StellCode").getLength() == 1)
          {
            int code = data.getArray("StellCode").getItem(0).getScaledValue("Code").intValue();
            
            m_debug.finest("Code " + code);

            WvzInhaltUmsetzungTls wvzInhaltUmsetzungTls = (WvzInhaltUmsetzungTls)mapWvzInhalt.get(code);
            
            if (wvzInhaltUmsetzungTls != null)
            {
              dataDirigentEigenschaftIst.getItem("Eigenschaft").getReferenceValue("AnzeigeInhalt").setText(wvzInhaltUmsetzungTls.getPidWvzInhalt());
            }
            
            tlsWzgWvzStellZustandBlinken(data.getArray("WVZAnzeige").getItem(0), dataDirigentEigenschaftIst);
            
            dataDirigentEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(dirigentPid, "tlsWzgStellZustand", STATUS_OK));
          }
          else
          {
            tlsWzgWechselTextBlinken(null, dataDirigentEigenschaftIst);

            dataDirigentEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(dirigentPid, "tlsWzgStellZustand", STATUS_UNDEFINIERT));
          }
        }
      }
      else // keine Daten vorhanden
      {
        tlsWzgWechselTextBlinken(null, dataDirigentEigenschaftIst);

        dataDirigentEigenschaftIst.getScaledValue("Status").setText(statusBestimmen(dirigentPid, "tlsWzgStellZustand", STATUS_UNDEFINIERT));
      }
    }
    
    return dataDirigentEigenschaftIst;
  }
  
  /**
   * Abarbeitung der Befehle aus dem Modell AnzeigenGlobal.
   * 
   * @param resultData AttributGruppe atg.sirigentEigenschaftSoll.
   * @param zeitpunktUebergeordnet Zeitpunkt der bergeordneten Schaltung.
   * @return Befehl fr das Modell KExTls.
   */
  public ResultData dirigentEigenschaftSoll(ResultData resultData, Long zeitpunktUebergeordnet)
  {
    ResultData ergebnis = null;
    
    SystemObject objekt = resultData.getObject();
    
    DirigentAktorTls dirigentAktor = (DirigentAktorTls)getListeDirigentAktorTlsDirigent().get(objekt.getPid());

    if (dirigentAktor == null)
      return null;
    
    if (resultData.isNoSourceAvailable()) // keine Quelle vorhanden
    {
      return null;
    }
    else // Quelle vorhanden
    {
      if (resultData.hasData()) // Daten vorhanden
      {
        Data data = resultData.getData();
    
        DirigentUmsetzungTls dirigentUmsetzungTls = dirigentAktor.getDirigentUmsetzung();
    
        long zeitpunkt = data.getTimeValue("SollZeitpunkt").getMillis();

        // BefehlsArt prfen
        
        int befehlsArt = AnzeigeSollBefehlsArt.eigenschaftBefehlsArtBestimmen(data);
        
        if (befehlsArt != AnzeigeSollBefehlsArt.WvzInhalt)
        {
          m_debug.error("Anzeige " + resultData.getObject().getPid() + " kann keine Stellcode-Ansteuerung");
            
          return null;
        }
        else
        {
          m_debug.error("ATG fr Dirigent " + resultData.getObject().getPid() + " beinhaltet Befehl");
        }
        
        if (befehlsArt == AnzeigeSollBefehlsArt.WvzInhalt) // reine Stellcode-Ansteuerung
        {
          HashMap mapWvzInhalt = null;
          
          ConfigurationObject objektAnzeigeInhalt = (ConfigurationObject)data.getItem("Eigenschaft").getReferenceValue("AnzeigeInhalt").getSystemObject();
          
          AttributeGroup atgWvzInhalt = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup("atg.wvzInhalt");
          
          Data dataWvzInhalt = objektAnzeigeInhalt.getConfigurationData(atgWvzInhalt);
          String anzeigeInhalt = dataWvzInhalt.getTextValue("Bildinhalt").getValueText();
          
          m_debug.finest("AnzeigeInhalt " + anzeigeInhalt);

          // DEs bestimmen
          
          mapWvzInhalt = dirigentUmsetzungTls.getListeWvzInhaltUmsetzungTlsWvzInhalt();
            
          WvzInhaltUmsetzungTls wvzInhaltUmsetzungTls = (WvzInhaltUmsetzungTls)mapWvzInhalt.get(anzeigeInhalt);
  
          int code = 0;
            
          if (wvzInhaltUmsetzungTls != null)
          {
            code = wvzInhaltUmsetzungTls.getCode();
          }
          else
          {
            m_debug.error("Objekt " + resultData.getObject().getPid() + " WvzInhalt " + anzeigeInhalt + " nicht gefunden");
          }
            
          SystemObject de = dirigentAktor.getDe();
          
          int anzahlGlobalUmsetzung = dirigentUmsetzungTls.getListeGlobalUmsetzung().size();

          for (int j = 0; j < anzahlGlobalUmsetzung; j++)
          {
            GlobalUmsetzung globalUmsetzung = (GlobalUmsetzung)dirigentUmsetzungTls.getListeGlobalUmsetzung().get(j);

            for (int k = 0; k < globalUmsetzung.getVerwendeteAttributGruppenAspekte().getAttributGruppenAspekte().size(); k++)
            {
              GlobalUmsetzungVerwendeteAttributGruppenAspekte guvaga = globalUmsetzung.getVerwendeteAttributGruppenAspekte();
              
              String atg = guvaga.getAttributGruppenAspekte().get(k).getPidAttributGruppe();
              String asp = guvaga.getAttributGruppenAspekte().get(k).getPidSendAspekt();
              String anzeigePrinzip = guvaga.getAttributGruppenAspekte().get(k).getAnzeigePrinzip();

              if (asp.length() > 0)
              {
                if ((atg.equals("atg.tlsWzgWvzStellZustand")) ||
                    (atg.equals("atg.tlsWzgStellZustand")))
                {
                  AttributeGroup atgErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup(atg);
                  Aspect aspErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAspect(asp);
                  
                  Data dataErgebnis = AnzeigenGlobalZwischenSchichtTls.getConnection().createData(AnzeigenGlobalZwischenSchichtTls.getConnection().getDataModel().getAttributeGroup(atg));

                  if (atg.equals("atg.tlsWzgWvzStellZustand"))
                  {
                    dataErgebnis.getUnscaledValue("WvzCode").set(code);

                    // BlinkenStatus
                    
                    dataErgebnis.getItem("Blinken").getScaledValue("Status").setText("aus");
                    
                    // BlinkenDauer
                    
                    dataErgebnis.getItem("Blinken").getScaledValue("Dauer").setText("aus");
                    
                    if (zeitpunkt < ((new Date()).getTime()-600000000))
                    {
                      zeitpunkt = zeitpunktUebergeordnet.longValue();
                    }
                    
                    ergebnis = new ResultData(de, new DataDescription(atgErgebnis, aspErgebnis), zeitpunkt, dataErgebnis);
                  }
                  
                  if (atg.equals("atg.tlsWzgStellZustand"))
                  {
                    dataErgebnis.getScaledValue("AnzeigePrinzip").setText(anzeigePrinzip);
                    dataErgebnis.getArray("WVZAnzeige").setLength(0);
                    dataErgebnis.getArray("StellCode").setLength(1);
                    dataErgebnis.getArray("TextDefinition").setLength(0);
                    dataErgebnis.getArray("GruppenDefinition").setLength(0);
                    
                    dataErgebnis.getArray("StellCode").getItem(0).getUnscaledValue("Code").set(code);

                    // BlinkenStatus
                    
                    dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Status").setText("aus");

                    // BlinkenDauer
                    
                    dataErgebnis.getArray("StellCode").getItem(0).getItem("Blinken").getScaledValue("Dauer").setText("aus");

                    dataErgebnis.getArray("StellCode").getItem(0).getItem("Status").getScaledValue("Fehler").setText("ok");
                    dataErgebnis.getArray("StellCode").getItem(0).getItem("Status").getScaledValue("Programm").setText("abgeschlossen");
                    
                    if (zeitpunkt < ((new Date()).getTime()-600000000))
                    {
                      zeitpunkt = zeitpunktUebergeordnet.longValue();
                    }
                    
                    ergebnis = new ResultData(de, new DataDescription(atgErgebnis, aspErgebnis), zeitpunkt, dataErgebnis);
                  }
                  
                  break;
                }
              }
            }
          }
        }
      } 
      else // keine Daten vorhanden
      {
        return null;
      }
    }
    
    return ergebnis;
  }
  
  /**
   * Abarbeitung des Anteiles Blinken des TLS-WechselText genau nach TLS 93.
   * 
   * @param data Daten tlsWzgBlinken.
   * @param dataAnzeigeEigenschaftIst Daten AnzeigeEigenschaftIst.
   * @return Daten AnzeigeEigenschaftIst mit eingearbeiteten Blinkinformation.
   */
  protected Data tlsWzgWechselTextBlinken(Data data, Data dataAnzeigeEigenschaftIst)
  {
    return tlsWzgBlinken(data, dataAnzeigeEigenschaftIst);
  }
  
  /**
   * Abarbeitung des Anteiles Blinken des TLS-WvzStellZustand genau nach TLS 93.
   * 
   * @param data Daten tlsWzgBlinken.
   * @param dataAnzeigeEigenschaftIst Daten AnzeigeEigenschaftIst.
   * @return Daten AnzeigeEigenschaftIst mit eingearbeiteten Blinkinformation.
   */
  protected Data tlsWzgWvzStellZustandBlinken(Data data, Data dataAnzeigeEigenschaftIst)
  {
    return tlsWzgBlinken(data, dataAnzeigeEigenschaftIst);
  }
  
  /**
   * Abarbeitung des Anteiles Blinken genau nach TLS 93.
   * 
   * @param data Daten tlsWzgBlinken.
   * @param dataAnzeigeEigenschaftIst Daten AnzeigeEigenschaftIst.
   * @return Daten AnzeigeEigenschaftIst mit eingearbeiteten Blinkinformation.
   */
  protected Data tlsWzgBlinken(Data data, Data dataAnzeigeEigenschaftIst)
  {
    dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getScaledValue("Blinken").setText("Nein");
    dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getScaledValue("Blinktakt").setText("Undefiniert");

    if (data != null)
    {
      String blinkenStatusTls = data.getItem("Blinken").getTextValue("Status").getValueText();
      
      if (blinkenStatusTls.equals("ein"))
      {
        int blinkDauerTls = data.getItem("Blinken").getUnscaledValue("Dauer").intValue();
        int blinkDauerAq = blinkDauerTls*200; // Faktor 200 ms

        dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getScaledValue("Blinken").setText("Ja");
        dataAnzeigeEigenschaftIst.getItem("Eigenschaft").getUnscaledValue("Blinktakt").set(blinkDauerAq);
      }
    }
    
    return dataAnzeigeEigenschaftIst;
  }
  
  /**
   * Holen der HashMap mit DirigentAktorTls mit Key Dirigent.
   * 
   * @return HashMap mit DirigentAktorTls mit Key Dirigent.
   */
  public HashMap getListeDirigentAktorTlsDirigent()
  {
    return m_listeDirigentAktorTlsDirigent;
  }

  /**
   * Holen der HashMap mit DirigentAktorTls mit Key DE's.
   * 
   * @return HashMap mit DirigentAktorTls mit Key DE's.
   */
  public HashMap getListeDirigentAktorTlsDes()
  {
    return m_listeDirigentAktorTlsDes;
  }

  /**
   * Holen der HashMap mit Attributgruppe atg.dirigentEigenschaftIst fr alle Dirigenten aktuell.
   * 
   * @return HashMap mit Attributgruppe atg.dirigentEigenschaftIst fr alle Dirigenten aktuell.
   */
  public HashMap getListeDirigentEigenschaftIst()
  {
    return m_listeDirigentEigenschaftIst;
  }
  
  /**
   * Holen der HashMap fr Status Methoden fr alle Dirigenten.
   * 
   * @return HashMap fr Status Methoden fr alle Dirigenten.
   */
  public HashMap getListeDirigentStatus()
  {
    return m_listeDirigentStatus;
  }

  /**
   * Bestimmen des aktuellen Dirigenten-Status mit der Mglichkeit gleichzeitiger bergabe des Status einer Abarbeitungs-Methode.
   * 
   * @param dirigentPid Pid des Dirigent.
   * @param methode Abarbeitungsmethode oder <code>null</code>, wenn nur Bestimmung des aktuellen Status.
   * @param status aktuelle Status der Abarbeitungsmethode, wenn methode nicht <code>null</code>. 
   * @return aktueller Dirigenten-Status, bestimmt aus allen fr den Dirigenten vorhandenen Methoden.
   */
  @SuppressWarnings("unchecked")
  protected String statusBestimmen(String dirigentPid, String methode, int status)
  {
//    System.out.println("statusBestimmen");
    
    HashMap<String, Integer> statusMethoden = (HashMap<String, Integer>)getListeDirigentStatus().get(dirigentPid);
    
    if (statusMethoden == null)
    {
      statusMethoden = new HashMap<String, Integer>();
      
      getListeDirigentStatus().put(dirigentPid, statusMethoden);
    }

    if (methode != null)
    {
      boolean bernehmen = true;
      
      if (bernehmen)
      {
        statusMethoden.put(methode, status); 
      }
    }
    
    Set<String> set = statusMethoden.keySet();
    Iterator iterator = set.iterator();
      
    int wert = 0;
    int wertMax = STATUS_OK;
    
    while (iterator.hasNext())
    {
      String meth = (String)iterator.next();
      
      wert = statusMethoden.get(meth);
      
//      System.out.println("Methode " + meth + " Wert " + wert);
      
      wertMax = (wert > wertMax) ? wert : wertMax;
    }
    
    return statusAusWertBestimmen(wertMax);
  }
  
  /**
   * Umsetzung des kodierten Status in den entsprechenden Text.
   * 
   * @param status kodierter Status.
   * @return dem Status entsprechender Text.
   */
  protected String statusAusWertBestimmen(int status)
  {
    String ergebnis = "OK";
    
    switch (status)
    {
      case STATUS_GESTOERT:
      {
        ergebnis = "Gestrt";

        break;
      }
      
      case STATUS_KOMMUNIKATIONSAUSFALL:
      {
        ergebnis = "Kommunikationsausfall";

        break;
      }
      
      case STATUS_STROMAUSFALL:
      {
        ergebnis = "Stromausfall";

        break;
      }
      
      case STATUS_TEILSTOERUNG:
      {
        ergebnis = "Teilstrung";

        break;
      }
      
      case STATUS_TRKONTAKT:
      {
        ergebnis = "Trkontakt";

        break;
      }
      
      case STATUS_UNDEFINIERT:
      {
        ergebnis = "Undefiniert";

        break;
      }
      
      default:
      {
        break;
      }
    }
    
    return ergebnis;
  }
}
