/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.agzs.tls;

import java.util.Vector;

import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Klasse fr die zerlegte Speicherung der verwendeten AttributGruppen-Aspekt-Kombinationen.<br>
 * Die Information besteht maximal aus "AttributGruppe:EmpfangAspekt:SendAspekt;AnzeigePrinzip"<br>
 * Alle Informationen, ausser <AttributGruppe> sind optional.<br>
 * Ist nur die AttributGruppe angegeben, so wird EmpfangAspekt auf "asp.tlsZustand" und SendAspekt auf "asp.tlsVorgabe" gesetzt.
 * Ist AttributGruppe und EmpfangAspekt ohne SendAspekt angegeben, so wird bei EmpfangAspekt "asp.tlsZustand" der SendAspekt automatisch auf "asp.tlsVorgabe" gesetzt, sonst bleibt er auf "".<br>
 * <br>
 * Ist kein <AnzeigePrinzip> angegeben, so wird "Cluster" gesetzt.
 * Folgende Angaben sind bei <AnzeigePrinzip> mglich: 
 * "a", "b", "c", "d", "e", "Cluster"<br>
 * <br>
 * Beispiel: "atg.tlsWzgStellZustand:asp.tlsZustand:asp.tlsVorgabe;b"<br>
 *
 * @version $Revision: 1.1 $ / $Date: 2008/09/15 08:37:49 $ / ($Author: Drapp $)
 *
 * @author Dambach-Werke GmbH
 * @author Thomas Drapp
 *
 */
public class GlobalUmsetzungVerwendeteAttributGruppenAspekte
{
  /**
   * DebugLogger fr Debug-Ausgaben.
   */
  private static Debug m_debug = Debug.getLogger();
  /**
   * Liste mit allen verwendeten AttributGruppen-Aspekt-Kombinationen.
   */
  protected Vector<AttributGruppenAspekte> m_attributGruppenAspekte = new Vector<AttributGruppenAspekte>();
  
  /**
   * Konstruktor mit bergabe eines Text-Array mit den versorgten VerwendeteAttributGruppenAspekte.
   * 
   * @param array Text-Array mit den versorgten VerwendeteAttributGruppenAspekte.
   */
  public GlobalUmsetzungVerwendeteAttributGruppenAspekte(TextArray array)
  {
    long anzahl = array.getLength();
    
    for (int i = 0; i < anzahl; i++)
    {
      AttributGruppenAspekte attributGruppenAspekt = new AttributGruppenAspekte(array.getText(i));
      
      getAttributGruppenAspekte().add(attributGruppenAspekt);
    }
  }

  /**
   * Holen der AttributGruppen-Aspekt-AnzeigePrinzip-Kombination mit dem bergebenen Index.
   * 
   * @return AttributGruppen-Aspekt-AnzeigePrinzip-Kombination.
   */
  public AttributGruppenAspekte getAttributGruppenAspekteIndex(int i)
  {
    return m_attributGruppenAspekte.get(i);
  }

  /**
   * Holen der Liste aller versorgten AttributGruppen-Aspekt-AnzeigePrinzip-Kombinationen.
   * 
   * @return Liste aller versorgten AttributGruppen-Aspekt-AnzeigePrinzip-Kombinationen.
   */
  public Vector<AttributGruppenAspekte> getAttributGruppenAspekte()
  {
    return m_attributGruppenAspekte;
  }

  /**
   * Setzen der Liste aller versorgten AttributGruppen-Aspekt-AnzeigePrinzip-Kombinationen.
   * 
   * @param verwendeteAttributGruppenAspekte Liste aller versorgten AttributGruppen-Aspekt-AnzeigePrinzip-Kombinationen.
   */
  public void setAttributGruppenAspekte(Vector<AttributGruppenAspekte> verwendeteAttributGruppenAspekte)
  {
    m_attributGruppenAspekte = verwendeteAttributGruppenAspekte;
  }

  /**
   * Klasse fr die interne Speicherung einer AttributGruppen-Aspekt-AnzeigePrinzip-Kombination.
   */
  class AttributGruppenAspekte
  {
    /**
     * Pid der AttributGruppe.
     */
    String m_pidAttributGruppe;
    /**
     * Pid des Empfangs-Aspekt.
     */
    String m_pidEmpfangAspekt;
    /**
     * Pid des Sende-Aspekt.
     */
    String m_pidSendAspekt;
    /**
     * AnzeigePrinzip fr Typ 55 usw. ("a", "b", "c", "d", "e" oder "Cluster")
     */
    String m_anzeigePrinzip;
    
    /**
     * Konstruktor mit noch aufzubereitenden Daten.
     * 
     * @param attributGruppeAspektAnzeigePrinzip Text zusammengesetzt aus Pid der AttributGruppe und Pid des Aspekt, getrennt durch ':' und dem AnzeigePrinzip getrennt durch ';'.
     */
    public AttributGruppenAspekte(String attributGruppeAspektAnzeigePrinzip)
    {
      String[] ddAnzeigePrinzip = attributGruppeAspektAnzeigePrinzip.split(";");

      String attributGruppeAspekt = "";
      
      if (ddAnzeigePrinzip.length > 0)
      {
        attributGruppeAspekt = ddAnzeigePrinzip[0];
      }
      
      if (ddAnzeigePrinzip.length > 1)
      {
        String anzeigePrinzip = ddAnzeigePrinzip[1];
        
        if (anzeigePrinzip.equalsIgnoreCase("a"))
        {
          setAnzeigePrinzip("a");
        } 
        else if (anzeigePrinzip.equalsIgnoreCase("b"))
        {
          setAnzeigePrinzip("b");
        }
        else if (anzeigePrinzip.equalsIgnoreCase("c"))
        {
          setAnzeigePrinzip("c");
        }
        else if (anzeigePrinzip.equalsIgnoreCase("d"))
        {
          setAnzeigePrinzip("d");
        }
        else if (anzeigePrinzip.equalsIgnoreCase("e"))
        {
          setAnzeigePrinzip("e");
        }
        else
        {
          setAnzeigePrinzip("Cluster");
        }
      }
      else
      {
        setAnzeigePrinzip("Cluster");
      }
      
      String[] werte = attributGruppeAspekt.split(":");
      
      if (werte.length == 1)
      {
        setPidAttributGruppe(werte[0]);
        setPidEmpfangAspekt("asp.tlsZustand"); // default Rckmeldung Schaltung
        setPidSendAspekt("asp.tlsVorgabe"); // default Befehl Schaltung
      }
      else if (werte.length == 2)
      {
        setPidAttributGruppe(werte[0]);
        setPidEmpfangAspekt(werte[1]);
        
        if (getPidEmpfangAspekt().equals("asp.tlsZustand"))
        {
          setPidSendAspekt("asp.tlsVorgabe"); // default Befehl Schaltung
        }
        else
        {
          setPidSendAspekt("");
        }
      }
      else if (werte.length == 3)
      {
        setPidAttributGruppe(werte[0]);
        setPidEmpfangAspekt(werte[1]);
        setPidSendAspekt(werte[2]);   
      }
/*
      else if (werte.length == 4) // TODO nach Konfig-Aenderung wieder entfernen
      {
        setPidAttributGruppe(werte[0]);
        setPidEmpfangAspekt(werte[1]);
        setPidSendAspekt(werte[2]);
        setAnzeigePrinzip(werte[3]);
      }
*/
      else
      {
        m_debug.config("AttributGruppenAspekteAnzeigePrinzip fehlerhaft '" + attributGruppeAspektAnzeigePrinzip + "'");
      }
    }

    /**
     * Konstruktor mit aufbereiteten Daten (AnzeigePrinzip "Cluster").
     * 
     * @param pidAttributGruppe Pid der AttributGruppe.
     * @param pidEmpfangAspekt Pid des Empfangs-Aspekt.
     * @param pidSendAspekt Pid des Sende-Aspekt.
     */
    public AttributGruppenAspekte(String pidAttributGruppe, String pidEmpfangAspekt, String pidSendAspekt)
    {
      setPidAttributGruppe(pidAttributGruppe);
      setPidEmpfangAspekt(pidEmpfangAspekt);
      setPidSendAspekt(pidSendAspekt);
      setAnzeigePrinzip("Cluster");
    }

    /**
     * Konstruktor mit aufbereiteten Daten.
     * 
     * @param pidAttributGruppe Pid der AttributGruppe.
     * @param pidEmpfangAspekt Pid des Empfangs-Aspekt.
     * @param pidSendAspekt Pid des Sende-Aspekt.
     */
    public AttributGruppenAspekte(String pidAttributGruppe, String pidEmpfangAspekt, String pidSendAspekt, String anzeigePrinzip)
    {
      setPidAttributGruppe(pidAttributGruppe);
      setPidEmpfangAspekt(pidEmpfangAspekt);
      setPidSendAspekt(pidSendAspekt);
      setAnzeigePrinzip(anzeigePrinzip);
    }

    /**
     * Holen der Pid des Empfangs-Aspekt.
     * 
     * @return Pid des Empfangs-Aspekt.
     */
    public String getPidEmpfangAspekt()
    {
      return m_pidEmpfangAspekt;
    }

    /**
     * Setzen der Pid des Empfangs-Aspekt.
     * 
     * @param pidEmpfangAspekt Pid des Empfangs-Aspekt.
     */
    public void setPidEmpfangAspekt(String pidEmpfangAspekt)
    {
      m_pidEmpfangAspekt = pidEmpfangAspekt;

//      System.out.println("PidEmpfangAspekt " + m_pidEmpfangAspekt);
    }

    /**
     * Holen der Pid des Sende-Aspekt.
     * 
     * @return Pid des Sende-Aspekt.
     */
    public String getPidSendAspekt()
    {
      return m_pidSendAspekt;
    }

    /**
     * Setzen der Pid des Sende-Aspekt.
     * 
     * @param pidSendAspekt Pid des Sende-Aspekt.
     */
    public void setPidSendAspekt(String pidSendAspekt)
    {
      m_pidSendAspekt = pidSendAspekt;

//      System.out.println("PidSendAspekt " + m_pidSendAspekt);
    }

    /**
     * Holen der Pid der AttributGruppe.
     * 
     * @return Pid der AttributGruppe.
     */
    public String getPidAttributGruppe()
    {
      return m_pidAttributGruppe;
    }

    /**
     * Setzen der Pid der AttributGruppe.
     * 
     * @param pidAttributGruppe Pid der AttributGruppe.
     */
    public void setPidAttributGruppe(String pidAttributGruppe)
    {
      m_pidAttributGruppe = pidAttributGruppe;
      
//      System.out.println("PidAttributGruppe " + m_pidAttributGruppe);
    }

    /**
     * @return Returns the anzeigePrinzip.
     */
    public String getAnzeigePrinzip()
    {
      return m_anzeigePrinzip;
    }

    /**
     * @param anzeigePrinzip The anzeigePrinzip to set.
     */
    public void setAnzeigePrinzip(String anzeigePrinzip)
    {
      if (anzeigePrinzip.equalsIgnoreCase("a"))
      {
        m_anzeigePrinzip = "a";
      } 
      else if (anzeigePrinzip.equalsIgnoreCase("b"))
      {
        m_anzeigePrinzip = "b";
      }
      else if (anzeigePrinzip.equalsIgnoreCase("c"))
      {
        m_anzeigePrinzip = "c";
      }
      else if (anzeigePrinzip.equalsIgnoreCase("d"))
      {
        m_anzeigePrinzip = "d";
      }
      else if (anzeigePrinzip.equalsIgnoreCase("e"))
      {
        m_anzeigePrinzip = "e";
      }
      else
      {
        m_anzeigePrinzip = "Cluster";
      }
    }
  }
}
